@@ -402,6 +402,7 @@ int setup_env(env_t *env, bool preserve_env)
 	    (env->stepid != SLURM_INTERACTIVE_STEP)) {
 		char *str_verbose, *str_bind1 = NULL, *str_bind2 = NULL;
 		char *str_bind_list, *str_bind_type = NULL, *str_bind = NULL;
+		bool append_cpu_bind = false;
 
 		unsetenvp(env->env, "SLURM_CPU_BIND");
 		unsetenvp(env->env, "SLURM_CPU_BIND_LIST");
@@ -427,17 +428,21 @@ int setup_env(env_t *env, bool preserve_env)
 			str_bind2 = "none";
 		} else if (env->cpu_bind_type & CPU_BIND_MAP) {
 			str_bind2 = "map_cpu:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_MASK) {
 			str_bind2 = "mask_cpu:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_LDRANK) {
 			str_bind2 = "rank_ldom";
 		} else if (env->cpu_bind_type & CPU_BIND_LDMAP) {
 			str_bind2 = "map_ldom:";
+			append_cpu_bind = true;
 		} else if (env->cpu_bind_type & CPU_BIND_LDMASK) {
 			str_bind2 = "mask_ldom:";
+			append_cpu_bind = true;
 		}
 
-		if (env->cpu_bind)
+		if (env->cpu_bind && append_cpu_bind)
 			str_bind_list = env->cpu_bind;
 		else
 			str_bind_list = "";
@@ -1236,6 +1241,7 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	slurm_step_layout_req_t step_layout_req;
 	uint16_t cpus_per_task_array[1];
 	uint32_t cpus_task_reps[1];
+	char *tres_per_task = NULL;
 
 	if (!batch)
 		return SLURM_ERROR;
@@ -1318,6 +1324,15 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	if (getenvp(*dest, "SLURM_CPUS_PER_TASK"))
 		env_array_overwrite_fmt(dest, "SLURM_CPUS_PER_TASK", "%u",
 					cpus_per_task);
+	if ((tres_per_task = getenvp(*dest, "SLURM_TRES_PER_TASK")) &&
+	    xstrstr(tres_per_task, "cpu=")) {
+		char *new_tres_per_task = xstrdup(tres_per_task);
+		slurm_option_update_tres_per_task(cpus_per_task, "cpu",
+						  &new_tres_per_task);
+		env_array_overwrite_fmt(dest, "SLURM_TRES_PER_TASK", "%s",
+					new_tres_per_task);
+		xfree(new_tres_per_task);
+	}
 
 	if (step_layout_req.num_tasks) {
 		env_array_overwrite_fmt(dest, "SLURM_NTASKS", "%u",
