@@ -56,6 +56,8 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "slurm/slurm_errno.h"
+
 #include "src/common/assoc_mgr.h"
 #include "src/common/fd.h"
 #include "src/common/forward.h"
@@ -77,6 +79,8 @@
 #include "src/interfaces/hash.h"
 #include "src/interfaces/topology.h"
 
+#include "src/conmgr/conmgr.h"
+
 #include "src/slurmdbd/read_config.h"
 
 typedef struct {
@@ -102,7 +106,6 @@ static int message_timeout = -1;
 /* STATIC FUNCTIONS */
 static char *_global_auth_key(void);
 static void  _remap_slurmctld_errno(void);
-static uid_t _unpack_msg_uid(buf_t *buffer, uint16_t protocol_version);
 
 /* define slurmdbd_conf here so we can treat its existence as a flag */
 slurmdbd_conf_t *slurmdbd_conf = NULL;
@@ -636,16 +639,16 @@ slurm_get_srun_port_range(void)
 /* Change general slurm communication errors to slurmctld specific errors */
 static void _remap_slurmctld_errno(void)
 {
-	int err = slurm_get_errno();
+	int err = errno;
 
 	if (err == SLURM_COMMUNICATIONS_CONNECTION_ERROR)
-		slurm_seterrno(SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR);
+		errno = SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR;
 	else if (err ==  SLURM_COMMUNICATIONS_SEND_ERROR)
-		slurm_seterrno(SLURMCTLD_COMMUNICATIONS_SEND_ERROR);
+		errno = SLURMCTLD_COMMUNICATIONS_SEND_ERROR;
 	else if (err == SLURM_COMMUNICATIONS_RECEIVE_ERROR)
-		slurm_seterrno(SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR);
+		errno = SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR;
 	else if (err == SLURM_COMMUNICATIONS_SHUTDOWN_ERROR)
-		slurm_seterrno(SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR);
+		errno = SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR;
 }
 
 /**********************************************************************\
@@ -844,25 +847,17 @@ extern int slurm_unpack_received_msg(slurm_msg_t *msg, int fd, buf_t *buffer)
 		peer = fd_resolve_peer(fd);
 	}
 
-	if (unpack_header(&header, buffer) == SLURM_ERROR) {
-		rc = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
+	if ((rc = unpack_header(&header, buffer)))
 		goto total_return;
-	}
 
-	if (check_header_version(&header) < 0) {
-		uid_t uid = _unpack_msg_uid(buffer, header.version);
+	log_flag(NET_RAW, "%s: [%s] header version=0x%hx flags=0x%hx msg_type=%s(0x%hx) body_length=%ub ret_cnt=%hx forward.cnt=%hu forward.init=0x%hx forward.nodelist=%s forward.timeout=%u forward.tree_width=%hu orig_addr=%pA",
+		 __func__, peer, header.version, header.flags,
+		 rpc_num2string(header.msg_type), header.msg_type,
+		 header.body_length, header.ret_cnt, header.forward.cnt,
+		 header.forward.init, header.forward.nodelist,
+		 header.forward.timeout, header.forward.tree_width,
+		 &header.orig_addr);
 
-		/* peer may have not been resolved already */
-		if (!peer)
-			peer = fd_resolve_peer(fd);
-
-		error("%s: [%s] Invalid Protocol Version %u from uid=%u: %m",
-		      __func__, peer, header.version, uid);
-
-		rc = SLURM_PROTOCOL_VERSION_ERROR;
-		goto total_return;
-	}
-	//info("ret_cnt = %d",header.ret_cnt);
 	if (header.ret_cnt > 0) {
 		/* peer may have not been resolved already */
 		if (!peer)
@@ -887,6 +882,11 @@ extern int slurm_unpack_received_msg(slurm_msg_t *msg, int fd, buf_t *buffer)
 		xfree(header.forward.nodelist);
 	}
 
+	/* Copy over header info before attempting auth */
+	msg->protocol_version = header.version;
+	msg->msg_type = header.msg_type;
+	msg->flags = header.flags;
+
 	if (header.flags & SLURM_NO_AUTH_CRED)
 		goto skip_auth;
 
@@ -930,10 +930,6 @@ skip_auth:
 	/*
 	 * Unpack message body
 	 */
-	msg->protocol_version = header.version;
-	msg->msg_type = header.msg_type;
-	msg->flags = header.flags;
-
 	msg->body_offset =  get_buf_offset(buffer);
 
 	if ((header.body_length != remaining_buf(buffer)) ||
@@ -951,7 +947,7 @@ skip_auth:
 total_return:
 	destroy_forward(&header.forward);
 
-	slurm_seterrno(rc);
+	errno = rc;
 	if (rc != SLURM_SUCCESS) {
 		msg->auth_cred = NULL;
 		/* peer may have not been resolved already */
@@ -1057,7 +1053,7 @@ int slurm_receive_msg(int fd, slurm_msg_t *msg, int timeout)
 		FREE_NULL_BUFFER(buffer);
 
 endit:
-	slurm_seterrno(rc);
+	errno = rc;
 
 	/*
 	 * We just set errno, we need to return SLURM_ERROR if not SLURM_SUCCESS
@@ -1078,7 +1074,7 @@ endit:
  *		  forwarded the message to. List containing type
  *		  (ret_data_info_t).
  */
-List slurm_receive_msgs(int fd, int steps, int timeout)
+list_t *slurm_receive_msgs(int fd, int steps, int timeout)
 {
 	char *buf = NULL;
 	size_t buflen = 0;
@@ -1088,7 +1084,7 @@ List slurm_receive_msgs(int fd, int steps, int timeout)
 	slurm_msg_t msg;
 	buf_t *buffer;
 	ret_data_info_t *ret_data_info = NULL;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int orig_timeout = timeout;
 	char *peer = NULL;
 
@@ -1111,10 +1107,7 @@ List slurm_receive_msgs(int fd, int steps, int timeout)
 		orig_timeout = timeout;
 	}
 	if (steps) {
-		if (message_timeout < 0)
-			message_timeout = slurm_conf.msg_timeout * 1000;
-		orig_timeout = (timeout -
-				(message_timeout*(steps-1)))/steps;
+		orig_timeout = timeout / (2 * steps);
 		steps--;
 	}
 
@@ -1147,27 +1140,11 @@ List slurm_receive_msgs(int fd, int steps, int timeout)
 	log_flag_hex(NET_RAW, buf, buflen, "%s: [%s] read", __func__, peer);
 	buffer = create_buf(buf, buflen);
 
-	if (unpack_header(&header, buffer) == SLURM_ERROR) {
+	if ((rc = unpack_header(&header, buffer))) {
 		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
 		goto total_return;
 	}
 
-	if (check_header_version(&header) < 0) {
-		uid_t uid = _unpack_msg_uid(buffer, header.version);
-
-		/* peer may have not been resolved already */
-		if (!peer)
-			peer = fd_resolve_peer(fd);
-
-		error("%s: [%s] Invalid Protocol Version %u from uid=%u: %m",
-		      __func__, peer, header.version, uid);
-
-		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_PROTOCOL_VERSION_ERROR;
-		goto total_return;
-	}
-	//info("ret_cnt = %d",header.ret_cnt);
 	if (header.ret_cnt > 0) {
 		if (header.ret_list)
 			ret_list = header.ret_list;
@@ -1281,7 +1258,7 @@ total_return:
 
 }
 
-extern List slurm_receive_resp_msgs(int fd, int steps, int timeout)
+extern list_t *slurm_receive_resp_msgs(int fd, int steps, int timeout)
 {
 	char *buf = NULL;
 	size_t buflen = 0;
@@ -1291,7 +1268,7 @@ extern List slurm_receive_resp_msgs(int fd, int steps, int timeout)
 	slurm_msg_t msg;
 	buf_t *buffer;
 	ret_data_info_t *ret_data_info = NULL;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int orig_timeout = timeout;
 	char *peer = NULL;
 
@@ -1315,10 +1292,7 @@ extern List slurm_receive_resp_msgs(int fd, int steps, int timeout)
 	}
 
 	if (steps) {
-		if (message_timeout < 0)
-			message_timeout = slurm_conf.msg_timeout * 1000;
-		orig_timeout = timeout - (message_timeout * (steps - 1));
-		orig_timeout /= steps;
+		orig_timeout = timeout / (2 * steps);
 		steps--;
 	}
 
@@ -1351,22 +1325,8 @@ extern List slurm_receive_resp_msgs(int fd, int steps, int timeout)
 	log_flag_hex(NET_RAW, buf, buflen, "%s: [%s] read", __func__, peer);
 	buffer = create_buf(buf, buflen);
 
-	if (unpack_header(&header, buffer) == SLURM_ERROR) {
+	if ((rc = unpack_header(&header, buffer))) {
 		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
-		goto total_return;
-	}
-
-	if (check_header_version(&header) < 0) {
-		/* peer may have not been resolved already */
-		if (!peer)
-			peer = fd_resolve_peer(fd);
-
-		error("%s: [%s] Invalid Protocol Version %u: %m",
-		      __func__, peer, header.version);
-
-		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_PROTOCOL_VERSION_ERROR;
 		goto total_return;
 	}
 
@@ -1458,26 +1418,6 @@ total_return:
 	return ret_list;
 }
 
-/*
- * Try to determine the UID associated with a message with different
- * message header version, return INFINITE ((uid_t) -1) if we can't tell.
- */
-static uid_t _unpack_msg_uid(buf_t *buffer, uint16_t protocol_version)
-{
-	uid_t uid = INFINITE;
-	void *auth_cred = NULL;
-
-	if (!(auth_cred = auth_g_unpack(buffer, protocol_version)))
-		return uid;
-	if (auth_g_verify(auth_cred, slurm_conf.authinfo))
-		return uid;
-
-	uid = auth_g_get_uid(auth_cred);
-	auth_g_destroy(auth_cred);
-
-	return uid;
-}
-
 /*
  * NOTE: memory is allocated for the returned msg and the returned list
  *       both must be freed at some point using the slurm_free_functions
@@ -1543,26 +1483,11 @@ int slurm_receive_msg_and_forward(int fd, slurm_addr_t *orig_addr,
 	log_flag_hex(NET_RAW, buf, buflen, "%s: [%s] read", __func__, peer);
 	buffer = create_buf(buf, buflen);
 
-	if (unpack_header(&header, buffer) == SLURM_ERROR) {
+	if ((rc = unpack_header(&header, buffer))) {
 		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
 		goto total_return;
 	}
 
-	if (check_header_version(&header) < 0) {
-		uid_t uid = _unpack_msg_uid(buffer, header.version);
-
-		/* peer may have not been resolved already */
-		if (!peer)
-			peer = fd_resolve_peer(fd);
-
-		error("%s: [%s] Invalid Protocol Version %u from uid=%u: %m",
-		      __func__, peer, header.version, uid);
-
-		FREE_NULL_BUFFER(buffer);
-		rc = SLURM_PROTOCOL_VERSION_ERROR;
-		goto total_return;
-	}
 	if (header.ret_cnt > 0) {
 		/* peer may have not been resolved already */
 		if (!peer)
@@ -1688,7 +1613,7 @@ skip_auth:
 total_return:
 	destroy_forward(&header.forward);
 
-	slurm_seterrno(rc);
+	errno = rc;
 	if (rc != SLURM_SUCCESS) {
 		msg->msg_type = RESPONSE_FORWARD_FAILED;
 		msg->auth_cred = NULL;
@@ -1862,6 +1787,9 @@ extern int slurm_send_node_msg(int fd, slurm_msg_t *msg)
 		rc = slurm_persist_send_msg(msg->conn, buffer);
 		FREE_NULL_BUFFER(buffer);
 
+		if ((rc < 0) && (fd < 0))
+			fd = msg->conn->fd;
+
 		if ((rc < 0) && (errno == ENOTCONN)) {
 			if (slurm_conf.debug_flags & DEBUG_FLAG_NET)
 				peer = fd_resolve_peer(fd);
@@ -1971,7 +1899,7 @@ void slurm_get_ip_str(slurm_addr_t *addr, char *ip, unsigned int buf_len)
  */
 int slurm_get_peer_addr(int fd, slurm_addr_t * slurm_address)
 {
-	slurm_addr_t name;
+	slurm_addr_t name = {0};
 	socklen_t namelen = (socklen_t) sizeof(name);
 
 	if (getpeername((int) fd, (struct sockaddr *) &name, &namelen))
@@ -2029,8 +1957,14 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
-			  uint16_t msg_type, void *data)
+/**********************************************************************\
+ * simplified communication routines
+ * They open a connection do work then close the connection all within
+ * the function
+\**********************************************************************/
+
+static void _response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
+			   uint16_t msg_type, void *data)
 {
 	slurm_msg_t_init(resp_msg);
 	resp_msg->address = msg->address;
@@ -2071,11 +2005,46 @@ extern void response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
 	resp_msg->flags |= SLURM_NO_AUTH_CRED;
 }
 
-/**********************************************************************\
- * simplified communication routines
- * They open a connection do work then close the connection all within
- * the function
-\**********************************************************************/
+extern int send_msg_response(slurm_msg_t *source_msg, slurm_msg_type_t msg_type,
+			     void *data)
+{
+	int rc;
+	slurm_msg_t resp_msg;
+
+	if ((source_msg->conn_fd < 0) && !source_msg->conn &&
+	    !source_msg->conmgr_fd)
+		return ENOTCONN;
+
+	_response_init(&resp_msg, source_msg, msg_type, data);
+
+	if (source_msg->conmgr_fd) {
+		rc = conmgr_queue_write_msg(source_msg->conmgr_fd, &resp_msg);
+
+		if (rc)
+			log_flag(NET, "%s: [%s] write response RPC %s failure: %s",
+				 __func__,
+				 conmgr_fd_get_name(source_msg->conmgr_fd),
+				 rpc_num2string(msg_type), slurm_strerror(rc));
+
+		return rc;
+	}
+
+	resp_msg.conn_fd = source_msg->conn_fd;
+	resp_msg.conn = source_msg->conn;
+
+	rc = slurm_send_node_msg(source_msg->conn_fd, &resp_msg);
+
+	if (rc >= 0)
+		return SLURM_SUCCESS;
+
+	rc = errno;
+	log_flag(NET, "%s: [fd:%d] write response RPC %s failed: %s",
+		 __func__, (source_msg->conn ? source_msg->conn->fd :
+			    source_msg->conn_fd),
+		 rpc_num2string(msg_type), slurm_strerror(rc));
+
+	return rc;
+}
 
 /* slurm_send_rc_msg
  * given the original request message this function sends a
@@ -2085,19 +2054,16 @@ extern void response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
  */
 int slurm_send_rc_msg(slurm_msg_t *msg, int rc)
 {
-	slurm_msg_t resp_msg;
-	return_code_msg_t rc_msg;
+	return_code_msg_t rc_msg = {
+		.return_code = rc,
+	};
 
-	if (msg->conn_fd < 0) {
-		slurm_seterrno(ENOTCONN);
+	if ((rc = send_msg_response(msg, RESPONSE_SLURM_RC, &rc_msg))) {
+		errno = rc;
 		return SLURM_ERROR;
 	}
-	rc_msg.return_code = rc;
 
-	response_init(&resp_msg, msg, RESPONSE_SLURM_RC, &rc_msg);
-
-	/* send message */
-	return slurm_send_node_msg(msg->conn_fd, &resp_msg);
+	return SLURM_SUCCESS;
 }
 
 /* slurm_send_rc_err_msg
@@ -2109,20 +2075,17 @@ int slurm_send_rc_msg(slurm_msg_t *msg, int rc)
  */
 int slurm_send_rc_err_msg(slurm_msg_t *msg, int rc, char *err_msg)
 {
-	slurm_msg_t resp_msg;
-	return_code2_msg_t rc_msg;
+	return_code2_msg_t rc_msg = {
+		.return_code = rc,
+		.err_msg = err_msg,
+	};
 
-	if (msg->conn_fd < 0) {
-		slurm_seterrno(ENOTCONN);
+	if ((rc = send_msg_response(msg, RESPONSE_SLURM_RC_MSG, &rc_msg))) {
+		errno = rc;
 		return SLURM_ERROR;
 	}
-	rc_msg.return_code = rc;
-	rc_msg.err_msg     = err_msg;
 
-	response_init(&resp_msg, msg, RESPONSE_SLURM_RC_MSG, &rc_msg);
-
-	/* send message */
-	return slurm_send_node_msg(msg->conn_fd, &resp_msg);
+	return SLURM_SUCCESS;
 }
 
 /*
@@ -2136,23 +2099,19 @@ int slurm_send_reroute_msg(slurm_msg_t *msg,
 			   slurmdb_cluster_rec_t *cluster_rec,
 			   char *stepmgr)
 {
-	slurm_msg_t resp_msg;
-	reroute_msg_t reroute_msg = {0};
+	int rc;
+	reroute_msg_t reroute_msg = {
+		.working_cluster_rec = cluster_rec,
+		.stepmgr = stepmgr,
+	};
 
-	if (msg->conn_fd < 0) {
-		slurm_seterrno(ENOTCONN);
+	if ((rc = send_msg_response(msg, RESPONSE_SLURM_REROUTE_MSG,
+				    &reroute_msg))) {
+		errno = rc;
 		return SLURM_ERROR;
 	}
 
-	/* Don't free the cluster_rec, it's pointing to the actual object. */
-	reroute_msg.working_cluster_rec = cluster_rec;
-	reroute_msg.stepmgr = stepmgr;
-
-	response_init(&resp_msg, msg, RESPONSE_SLURM_REROUTE_MSG,
-			&reroute_msg);
-
-	/* send message */
-	return slurm_send_node_msg(msg->conn_fd, &resp_msg);
+	return SLURM_SUCCESS;
 }
 
 /*
@@ -2225,46 +2184,30 @@ _send_and_recv_msg(int fd, slurm_msg_t *req,
  *		  forwarded the message to. List containing type
  *		  (ret_data_info_t).
  */
-static List
-_send_and_recv_msgs(int fd, slurm_msg_t *req, int timeout)
+static list_t *_send_and_recv_msgs(int fd, slurm_msg_t *req, int timeout)
 {
-	List ret_list = NULL;
-	int steps = 0;
+	list_t *ret_list = NULL;
 
-	if (!req->forward.timeout) {
-		if (!timeout)
-			timeout = slurm_conf.msg_timeout * 1000;
-		req->forward.timeout = timeout;
-	}
-	if (slurm_send_node_msg(fd, req) >= 0) {
-		if (req->forward.cnt > 0) {
-			/* figure out where we are in the tree and set
-			 * the timeout for to wait for our children
-			 * correctly
-			 * (timeout+message_timeout sec per step)
-			 * to let the child timeout */
-			if (message_timeout < 0)
-				message_timeout =
-					slurm_conf.msg_timeout * 1000;
-			steps = req->forward.cnt + 1;
-			if (!req->forward.tree_width)
-				req->forward.tree_width =
-					slurm_conf.tree_width;
-			if (req->forward.tree_width)
-				steps /= req->forward.tree_width;
-			timeout = (message_timeout * steps);
-			steps++;
-
-			timeout += (req->forward.timeout*steps);
-		}
-		ret_list = slurm_receive_msgs(fd, steps, timeout);
-	}
+	if (slurm_send_node_msg(fd, req) >= 0)
+		ret_list = slurm_receive_msgs(fd, req->forward.tree_depth,
+					      req->forward.timeout);
 
 	(void) close(fd);
 
 	return ret_list;
 }
 
+static int _foreach_ret_list_hostname_set(void *x, void *arg)
+{
+	ret_data_info_t *ret_data_info = x;
+	char *name = arg;
+
+	if (!ret_data_info->node_name)
+		ret_data_info->node_name = xstrdup(name);
+
+	return 0;
+}
+
 /*
  * slurm_send_recv_controller_msg
  * opens a connection to the controller, sends the controller a message,
@@ -2315,6 +2258,10 @@ tryagain:
 			break;
 		}
 
+		if (comm_cluster_rec)
+			request_msg->protocol_version =
+				comm_cluster_rec->rpc_version;
+
 		rc = _send_and_recv_msg(fd, request_msg, response_msg, 0);
 		if (response_msg->auth_cred)
 			auth_g_destroy(response_msg->auth_cred);
@@ -2564,27 +2511,6 @@ again:
 	return rc;
 }
 
-/*
- * Open a connection to the "address" specified in the slurm msg `req'
- * Then, immediately close the connection w/out waiting for a reply.
- * Ignore any errors. This should only be used when you do not care if
- * the message is ever received.
- */
-void slurm_send_msg_maybe(slurm_msg_t *req)
-{
-	int fd = -1;
-
-	if ((fd = slurm_open_msg_conn(&req->address)) < 0) {
-		log_flag(NET, "%s: slurm_open_msg_conn(%pA): %m",
-			 __func__, &req->address);
-		return;
-	}
-
-	(void) slurm_send_node_msg(fd, req);
-
-	(void) close(fd);
-}
-
 /*
  *  Send a message to the nodelist specificed using fanout
  *    Then return List containing type (ret_data_info_t).
@@ -2595,9 +2521,9 @@ void slurm_send_msg_maybe(slurm_msg_t *req)
  *		    (if any) we forwarded the message to. List
  *		    containing type (ret_data_info_t).
  */
-List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout)
+list_t *slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout)
 {
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	hostlist_t *hl = NULL;
 
 	if (!running_in_daemon()) {
@@ -2637,14 +2563,12 @@ List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout)
  *		    (if any) we forwarded the message to. List
  *		    containing type (ret_types_t).
  */
-List slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name, int timeout)
+list_t *slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name, int timeout)
 {
 	static pthread_mutex_t conn_lock = PTHREAD_MUTEX_INITIALIZER;
 	static uint16_t conn_timeout = NO_VAL16, tcp_timeout = 2;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int fd = -1;
-	ret_data_info_t *ret_data_info = NULL;
-	list_itr_t *itr;
 	int i;
 
 	slurm_mutex_lock(&conn_lock);
@@ -2688,12 +2612,8 @@ List slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name, int timeout)
 		errno = SLURM_COMMUNICATIONS_CONNECTION_ERROR;
 		return ret_list;
 	} else {
-		itr = list_iterator_create(ret_list);
-		while ((ret_data_info = list_next(itr)))
-			if (!ret_data_info->node_name) {
-				ret_data_info->node_name = xstrdup(name);
-			}
-		list_iterator_destroy(itr);
+		(void) list_for_each(
+			ret_list, _foreach_ret_list_hostname_set, name);
 	}
 	return ret_list;
 }
@@ -2962,7 +2882,7 @@ extern int get_unit_type(char unit)
 extern int slurm_forward_data(
 	char **nodelist, char *address, uint32_t len, const char *data)
 {
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	int temp_rc = 0, rc = 0;
 	ret_data_info_t *ret_data_info = NULL;
 	slurm_msg_t msg;
