@@ -72,7 +72,6 @@
 
 /* bitstring.[ch] functions*/
 #define	bit_alloc		slurm_bit_alloc
-#define	bit_test		slurm_bit_test
 #define	bit_set			slurm_bit_set
 #define	bit_clear		slurm_bit_clear
 #define	bit_nclear		slurm_bit_nclear
@@ -88,7 +87,6 @@
 #define	bit_set_count		slurm_bit_set_count
 #define	bit_set_count_range	slurm_bit_set_count_range
 #define	bit_clear_count		slurm_bit_clear_count
-#define	bit_clear_count_range	slurm_bit_clear_count_range
 #define	bit_nset_max_count	slurm_bit_nset_max_count
 #define	bit_rotate_copy		slurm_bit_rotate_copy
 #define	bit_rotate		slurm_bit_rotate
@@ -99,8 +97,6 @@
 #define	bit_fmt_hexmask		slurm_bit_fmt_hexmask
 #define	bit_fmt_hexmask_trim	slurm_bit_fmt_hexmask_trim
 #define bit_unfmt_hexmask	slurm_bit_unfmt_hexmask
-#define	bit_fmt_binmask		slurm_bit_fmt_binmask
-#define bit_unfmt_binmask	slurm_bit_unfmt_binmask
 #define	bit_fls			slurm_bit_fls
 #define	bit_fls_from_bit	slurm_bit_fls_from_bit
 #define	bit_fill_gaps		slurm_bit_fill_gaps
@@ -118,11 +114,12 @@
 
 /* fd.[ch] functions */
 #define closeall		slurm_closeall
+#define closeall_except		slurm_closeall_except
 #define fd_set_blocking		slurm_fd_set_blocking
 #define fd_set_nonblocking	slurm_fd_set_nonblocking
 #define fd_get_socket_error	slurm_fd_get_socket_error
-#define send_fd_over_pipe	slurm_send_fd_over_pipe
-#define receive_fd_over_pipe	slurm_receive_fd_over_pipe
+#define send_fd_over_socket	slurm_send_fd_over_socket
+#define receive_fd_over_socket	slurm_receive_fd_over_socket
 #define rmdir_recursive		slurm_rmdir_recursive
 
 /* hostlist.[ch] functions */
@@ -267,6 +264,7 @@
 #define rehash_node		slurm_rehash_node
 #define hostlist2bitmap		slurm_hostlist2bitmap
 #define bitmap2node_name	slurm_bitmap2node_name
+#define node_name2bitmap	slurm_node_name2bitmap
 #define find_node_record	slurm_find_node_record
 
 /* pack.[ch] functions */
@@ -304,6 +302,8 @@
 #define	unpackstr_xmalloc	slurm_unpackstr_xmalloc
 #define	unpackstr_xmalloc_escaped slurm_unpackstr_xmalloc_escaped
 #define	unpackstr_xmalloc_chooser slurm_unpackstr_xmalloc_chooser
+#define packstr_func slurm_packstr_func
+#define safe_unpackstr_func slurm_safe_unpackstr_func
 #define	packstr_array		slurm_packstr_array
 #define	unpackstr_array		slurm_unpackstr_array
 #define	packmem_array		slurm_packmem_array
@@ -338,6 +338,9 @@
 #define get_extra_conf_path	slurm_get_extra_conf_path
 #define sort_key_pairs		slurm_sort_key_pairs
 #define conf_get_opt_str	slurm_conf_get_opt_str
+#define add_key_pair		slurm_add_key_pair
+#define add_key_pair_bool	slurm_add_key_pair_bool
+#define add_key_pair_own	slurm_add_key_pair_own
 
 /* run_in_daemon.[ch] functions */
 #define run_in_daemon           slurm_run_in_daemon
@@ -354,6 +357,11 @@
  * None exported today.
  * The header file used only for #define values. */
 
+/* state_save.[ch] functions */
+#define lock_state_files	slurm_lock_state_files
+#define unlock_state_files	slurm_unlock_state_files
+#define save_buf_to_state	slurm_save_buf_to_state
+
 /* strlcpy.[ch] functions */
 #ifndef HAVE_STRLCPY
 #define	strlcpy			slurm_strlcpy
@@ -391,6 +399,7 @@
 #define	_xmemcat		slurm_xmemcat
 #define	xstrdup			slurm_xstrdup
 #define	xstrdup_printf		slurm_xstrdup_printf
+#define _xstrdup_vprintf	slurm_xstrdup_vprintf
 #define	xstrndup		slurm_xstrndup
 #define	xbasename		slurm_xbasename
 #define	xdirname		slurm_xdirname
@@ -406,6 +415,7 @@
 #define xstrncasecmp		slurm_xstrncasecmp
 #define	xstrstr			slurm_xstrstr
 #define xstrcasestr		slurm_xstrcasestr
+#define xbase64_from_base64url	slurm_xbase64_from_base64url
 
 /* slurm_protocol_api.[ch] functions */
 #define convert_num_unit2       slurm_convert_num_unit2
@@ -417,8 +427,6 @@
 /* slurm_protocol_defs.[ch] functions */
 #define preempt_mode_string	slurm_preempt_mode_string
 #define preempt_mode_num	slurm_preempt_mode_num
-#define job_reason_string	slurm_job_reason_string
-#define job_reason_num		slurm_job_reason_num
 #define job_share_string	slurm_job_share_string
 #define job_state_string	slurm_job_state_string
 #define job_state_string_compact slurm_job_state_string_compact
@@ -434,7 +442,6 @@
 				slurm_node_state_string_complete
 #define private_data_string	slurm_private_data_string
 #define accounting_enforce_string slurm_accounting_enforce_string
-#define cray_nodelist2nids	slurm_cray_nodelist2nids
 #define reservation_flags_string slurm_reservation_flags_string
 #define print_multi_line_string slurm_print_multi_line_string
 
@@ -534,6 +541,17 @@
 /* topology.[ch] functions */
 #define topology_g_build_config slurm_topology_g_build_config
 
+/* job_state_reason.[ch] functions */
+#define job_state_reason_string	slurm_job_state_reason_string
+#define job_state_reason_num slurm_job_state_reason_num
+#define job_state_reason_check slurm_job_state_reason_check
+
+/* serializer.[ch] functions */
+#define serializer_g_init slurm_serializer_g_init
+#define serialize_g_data_to_string slurm_serialize_g_data_to_string
+#define serialize_g_string_to_data slurm_serialize_g_string_to_data
+#define serializer_g_fini slurm_serializer_g_fini
+
 #endif /* USE_ALIAS */
 
 #endif /*__SLURM_XLATOR_H__*/
