@@ -247,14 +247,14 @@ static bool _alter_table_after_upgrade(mysql_conn_t *mysql_conn,
 
 	/* check to see if upgrade has happened */
 	if (!have_value) {
-		const char *info;
+		const char *tmp_char;
 		char *query;
 		/*
 		 * confirm MariaDB is being used to avoid any ambiguity with
 		 * MySQL versions
 		 */
-		info = mysql_get_server_info(mysql_conn->db_conn);
-		if (xstrcasestr(info, "mariadb") &&
+		tmp_char = mysql_get_server_info(mysql_conn->db_conn);
+		if (xstrcasestr(tmp_char, "mariadb") &&
 		    (mysql_get_server_version(mysql_conn->db_conn) >= 100201)) {
 			query = "show columns from `qos_table` like 'preempt'";
 			result = mysql_db_query_ret(mysql_conn, query, 0);
@@ -294,8 +294,8 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	int i = 0;
-	List columns = NULL;
-	ListIterator itr = NULL;
+	list_t *columns = NULL;
+	list_itr_t *itr = NULL;
 	char *col = NULL;
 	int adding = 0;
 	int run_update = 0;
@@ -303,7 +303,7 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 	char *unique_index = NULL;
 	int old_primary = 0;
 	char *temp = NULL, *temp2 = NULL;
-	List keys_list = NULL;
+	list_t *keys_list = NULL;
 	db_key_t *db_key = NULL;
 
 	DEF_TIMERS;
@@ -1255,8 +1255,11 @@ extern void mysql_db_enable_streaming_replication(mysql_conn_t *mysql_conn)
 	uint64_t wsrep_on, wsrep_max_ws_size, fragment_size;
 
 	/* if this errors, assume wsrep_on doesn't exist, so must be disabled */
-	if (mysql_db_get_var_u64(mysql_conn, "wsrep_on", &wsrep_on))
+	if (mysql_db_get_var_u64(mysql_conn, "wsrep_on", &wsrep_on)) {
 		wsrep_on = 0;
+		if (errno == ER_UNKNOWN_SYSTEM_VARIABLE)
+			error("The prior error message regarding an undefined 'wsrep_on' variable is innocuous.  MySQL and MariaDB < 10.1 do not have this variable and Slurm will operate normally without it.");
+	}
 
 	debug2("wsrep_on=%"PRIu64, wsrep_on);
 
@@ -1342,16 +1345,16 @@ extern void mysql_db_restore_streaming_replication(mysql_conn_t *mysql_conn)
 {
 	int rc;
 	char *query;
-	uint64_t wsrep_on;
 
-	/* if this errors, assume wsrep_on doesn't exist, so must be disabled */
-	if (mysql_db_get_var_u64(mysql_conn, "wsrep_on", &wsrep_on))
-		wsrep_on = 0;
-
-	debug2("wsrep_on=%"PRIu64, wsrep_on);
-
-	if (!wsrep_on)
+	/*
+	 * Check if the conection has saved streaming replication settings.  If
+	 * not there is nothing to restore.
+	 */
+	if (!mysql_conn->wsrep_trx_fragment_unit_orig &&
+	    (mysql_conn->wsrep_trx_fragment_size_orig == NO_VAL64)) {
+		debug2("no streaming replication settings to restore");
 		return;
+	}
 
 	if (mysql_conn->wsrep_trx_fragment_unit_orig) {
 		query = xstrdup_printf(
