@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  hash.c - hash plugin driver
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC
- *  Written by Dominik Bartkiewicz <bart@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -80,6 +79,8 @@ extern int hash_g_init(void)
 {
 	int rc = SLURM_SUCCESS;
 	char *plugin_type = "hash";
+	char *hash_plugin_list = NULL, *plugin_list = NULL, *type = NULL;
+	char *save_ptr = NULL;
 
 	slurm_mutex_lock(&g_context_lock);
 
@@ -89,25 +90,47 @@ extern int hash_g_init(void)
 	g_context_num = 0;
 	memset(hash_id_to_inx, 0xff, HASH_PLUGIN_CNT);
 
-	xrecalloc(ops, g_context_num + 1, sizeof(slurm_ops_t));
-	xrecalloc(g_context, g_context_num + 1, sizeof(plugin_context_t *));
+	/* ensure k12 plugin is always loaded */
+	hash_plugin_list = xstrdup(slurm_conf.hash_plugin);
+	if (!xstrstr(hash_plugin_list, "k12"))
+		xstrcat(hash_plugin_list, ",k12");
+	plugin_list = hash_plugin_list;
 
-	g_context[g_context_num] = plugin_context_create(
-		plugin_type, "hash/k12", (void **)&ops[g_context_num],
-		syms, sizeof(syms));
-	if (!g_context[g_context_num] ||
-	    (*(ops[g_context_num].plugin_id) != HASH_PLUGIN_K12)) {
-		error("cannot create %s context for K12", plugin_type);
-		rc = SLURM_ERROR;
-		goto done;
+	while ((type = strtok_r(hash_plugin_list, ",", &save_ptr))) {
+		char *full_type = NULL;
+
+		xrecalloc(ops, g_context_num + 1, sizeof(slurm_ops_t));
+		xrecalloc(g_context, g_context_num + 1,
+			  sizeof(plugin_context_t *));
+
+		/* allow plugins to be specified as either "hash/k12" or "k12" */
+		if (!xstrncmp(type, "hash/", 5))
+			type += 5;
+		full_type = xstrdup_printf("hash/%s", type);
+
+		g_context[g_context_num] = plugin_context_create(
+			plugin_type, full_type, (void **) &ops[g_context_num],
+			syms, sizeof(syms));
+
+		if (!g_context[g_context_num]) {
+			error("cannot create %s context for %s",
+			      plugin_type, full_type);
+			rc = SLURM_ERROR;
+			xfree(full_type);
+			goto done;
+		}
+		xfree(full_type);
+
+		hash_id_to_inx[*(ops[g_context_num].plugin_id)] = g_context_num;
+		g_context_num++;
+		hash_plugin_list = NULL; /* for next iteration */
 	}
-	hash_id_to_inx[*(ops[g_context_num].plugin_id)] = g_context_num;
-	g_context_num++;
 
 	hash_id_to_inx[HASH_PLUGIN_DEFAULT] = 0;
 
 done:
 	slurm_mutex_unlock(&g_context_lock);
+	xfree(plugin_list);
 
 	return rc;
 }
