@@ -42,8 +42,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <dirent.h>
-
 #include "src/common/macros.h"
 #include "src/common/plugin.h"
 #include "src/common/plugrack.h"
@@ -53,44 +51,46 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
+/* opaque type */
+typedef struct switch_stepinfo switch_stepinfo_t;
+
 typedef struct slurm_switch_ops {
 	uint32_t     (*plugin_id);
-	int          (*state_save)        ( char *dir_name );
-	int          (*state_restore)     ( char *dir_name, bool recover );
-
-	int          (*alloc_jobinfo)     ( switch_jobinfo_t **jobinfo,
-					    uint32_t job_id, uint32_t step_id );
-	int          (*build_jobinfo)     ( switch_jobinfo_t *jobinfo,
-					    slurm_step_layout_t *step_layout,
-					    step_record_t *step_ptr );
-	int          (*duplicate_jobinfo) ( switch_jobinfo_t *source,
-					    switch_jobinfo_t **dest);
-	void         (*free_jobinfo)      ( switch_jobinfo_t *jobinfo );
-	int          (*pack_jobinfo)      ( switch_jobinfo_t *jobinfo,
+	int          (*state_save)        ( void );
+	int          (*state_restore)     ( bool recover );
+	void         (*pack_jobinfo)      ( void *switch_jobinfo,
 					    buf_t *buffer,
 					    uint16_t protocol_version );
-	int          (*unpack_jobinfo)    ( switch_jobinfo_t **jobinfo,
+	int          (*unpack_jobinfo)    ( void **switch_jobinfo,
+					    buf_t *buffer,
+					    uint16_t protocol_version );
+	int          (*build_stepinfo)    ( switch_stepinfo_t **stepinfo,
+					    slurm_step_layout_t *step_layout,
+					    step_record_t *step_ptr );
+	void         (*duplicate_stepinfo)( switch_stepinfo_t *source,
+					    switch_stepinfo_t **dest);
+	void         (*free_stepinfo)     ( switch_stepinfo_t *stepinfo );
+	void         (*pack_stepinfo)     ( switch_stepinfo_t *stepinfo,
+					    buf_t *buffer,
+					    uint16_t protocol_version );
+	int          (*unpack_stepinfo)   ( switch_stepinfo_t **stepinfo,
 					    buf_t *buffer,
 					    uint16_t protocol_version );
 	int          (*job_preinit)       ( stepd_step_rec_t *step );
 	int          (*job_init)          ( stepd_step_rec_t *step );
-	int          (*job_fini)          ( switch_jobinfo_t *jobinfo );
 	int          (*job_postfini)      ( stepd_step_rec_t *step);
-	int          (*job_attach)        ( switch_jobinfo_t *jobinfo,
+	int          (*job_attach)        ( switch_stepinfo_t *stepinfo,
 					    char ***env, uint32_t nodeid,
 					    uint32_t procid, uint32_t nnodes,
 					    uint32_t nprocs, uint32_t rank);
-	int          (*step_complete)     ( switch_jobinfo_t *jobinfo,
+	int          (*step_complete)     ( switch_stepinfo_t *stepinfo,
 					    char *nodelist );
-	int          (*step_allocated)    ( switch_jobinfo_t *jobinfo,
-					    char *nodelist );
-	int          (*state_clear)       ( void );
-	int          (*reconfig)          ( void );
-	int          (*job_step_pre_suspend)( stepd_step_rec_t *step );
-	int          (*job_step_post_suspend)( stepd_step_rec_t *step );
-	int          (*job_step_pre_resume)( stepd_step_rec_t *step );
-	int          (*job_step_post_resume)( stepd_step_rec_t *step );
-	void         (*job_complete)      ( uint32_t job_id );
+	void         (*job_start)         ( job_record_t *job_ptr );
+	void         (*job_complete)      ( job_record_t *job_ptr );
+	int          (*fs_init)           ( stepd_step_rec_t *step );
+	void         (*extern_stepinfo)   ( switch_stepinfo_t **stepinfo,
+					    job_record_t *job_ptr );
+	void	     (*extern_step_fini)  ( uint32_t job_id);
 } slurm_switch_ops_t;
 
 /*
@@ -99,28 +99,25 @@ typedef struct slurm_switch_ops {
  */
 static const char *syms[] = {
 	"plugin_id",
-	"switch_p_libstate_save",
-	"switch_p_libstate_restore",
-	"switch_p_alloc_jobinfo",
-	"switch_p_build_jobinfo",
-	"switch_p_duplicate_jobinfo",
-	"switch_p_free_jobinfo",
+	"switch_p_save",
+	"switch_p_restore",
 	"switch_p_pack_jobinfo",
 	"switch_p_unpack_jobinfo",
+	"switch_p_build_stepinfo",
+	"switch_p_duplicate_stepinfo",
+	"switch_p_free_stepinfo",
+	"switch_p_pack_stepinfo",
+	"switch_p_unpack_stepinfo",
 	"switch_p_job_preinit",
 	"switch_p_job_init",
-	"switch_p_job_fini",
 	"switch_p_job_postfini",
 	"switch_p_job_attach",
 	"switch_p_job_step_complete",
-	"switch_p_job_step_allocated",
-	"switch_p_libstate_clear",
-	"switch_p_reconfig",
-	"switch_p_job_step_pre_suspend",
-	"switch_p_job_step_post_suspend",
-	"switch_p_job_step_pre_resume",
-	"switch_p_job_step_post_resume",
+	"switch_p_job_start",
 	"switch_p_job_complete",
+	"switch_p_fs_init",
+	"switch_p_extern_stepinfo",
+	"switch_p_extern_step_fini",
 };
 
 static slurm_switch_ops_t  *ops            = NULL;
@@ -157,20 +154,20 @@ static int _load_plugins(void *x, void *arg)
 
 static dynamic_plugin_data_t *_create_dynamic_plugin_data(uint32_t plugin_id)
 {
-	dynamic_plugin_data_t *jobinfo_ptr = NULL;
+	dynamic_plugin_data_t *stepinfo_ptr = NULL;
 
-	jobinfo_ptr = xmalloc(sizeof(dynamic_plugin_data_t));
-	jobinfo_ptr->plugin_id = plugin_id;
+	stepinfo_ptr = xmalloc(sizeof(dynamic_plugin_data_t));
+	stepinfo_ptr->plugin_id = plugin_id;
 
-	return jobinfo_ptr;
+	return stepinfo_ptr;
 }
 
-extern int switch_init(bool only_default)
+extern int switch_g_init(bool only_default)
 {
 	int retval = SLURM_SUCCESS;
 	char *plugin_type = "switch";
 	int i, j, plugin_cnt;
-	List plugin_names = NULL;
+	list_t *plugin_names = NULL;
 	_plugin_args_t plugin_args = {0};
 
 	slurm_mutex_lock( &context_lock );
@@ -230,7 +227,7 @@ done:
 	return retval;
 }
 
-extern int switch_fini(void)
+extern int switch_g_fini(void)
 {
 	int rc = SLURM_SUCCESS, i;
 
@@ -249,87 +246,115 @@ fini:
 	return rc;
 }
 
-extern int  switch_g_reconfig(void)
+extern int switch_g_save(void)
 {
 	xassert(switch_context_cnt >= 0);
 
 	if (!switch_context_cnt)
 		return SLURM_SUCCESS;
 
-	return (*(ops[switch_context_default].reconfig))( );
+	return (*(ops[switch_context_default].state_save))();
 }
 
-extern int  switch_g_save(char *dir_name)
+extern int switch_g_restore(bool recover)
 {
 	xassert(switch_context_cnt >= 0);
 
 	if (!switch_context_cnt)
 		return SLURM_SUCCESS;
 
-	return (*(ops[switch_context_default].state_save))( dir_name );
+	return (*(ops[switch_context_default].state_restore))(recover);
 }
 
-extern int  switch_g_restore(char *dir_name, bool recover)
+/*
+ * These are designed so that the payload will be skipped if the plugin
+ * is unavailable.
+ */
+extern void switch_g_pack_jobinfo(void *switch_jobinfo, buf_t *buffer,
+				  uint16_t protocol_version)
 {
-	xassert(switch_context_cnt >= 0);
+	uint32_t length_position = 0, start = 0, end = 0;
+
+	length_position = get_buf_offset(buffer);
+	pack32(0, buffer);
 
 	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
+		return;
 
-	return (*(ops[switch_context_default].state_restore))
-		(dir_name, recover);
+	start = get_buf_offset(buffer);
+	pack32(*(ops[switch_context_default].plugin_id), buffer);
+	(*(ops[switch_context_default].pack_jobinfo))(switch_jobinfo, buffer,
+						      protocol_version);
+	end = get_buf_offset(buffer);
+	set_buf_offset(buffer, length_position);
+	pack32(end - start, buffer);
+	set_buf_offset(buffer, end);
 }
 
-extern int  switch_g_clear(void)
+extern int switch_g_unpack_jobinfo(void **switch_jobinfo, buf_t *buffer,
+				   uint16_t protocol_version)
 {
-	xassert(switch_context_cnt >= 0);
+	uint32_t length = 0, switch_jobinfo_end = 0;
+	uint32_t plugin_id = 0;
 
-	if (!switch_context_cnt)
+	safe_unpack32(&length, buffer);
+
+	if (remaining_buf(buffer) < length)
+		return SLURM_ERROR;
+
+	switch_jobinfo_end = get_buf_offset(buffer) + length;
+
+	if (!length || !switch_context_cnt) {
+		debug("%s: skipping switch_jobinfo data (%u)", __func__, length);
+		set_buf_offset(buffer, switch_jobinfo_end);
 		return SLURM_SUCCESS;
+	}
 
-	return (*(ops[switch_context_default].state_clear))( );
+	safe_unpack32(&plugin_id, buffer);
+
+	if (plugin_id != *(ops[switch_context_default].plugin_id)) {
+		debug("%s: skipping switch_jobinfo data", __func__);
+		set_buf_offset(buffer, switch_jobinfo_end);
+		return SLURM_SUCCESS;
+	}
+
+	if ((*(ops[switch_context_default].unpack_jobinfo))(switch_jobinfo,
+							    buffer,
+							    protocol_version))
+		goto unpack_error;
+
+	if (get_buf_offset(buffer) != switch_jobinfo_end) {
+		error("%s: plugin did not unpack until switch_jobinfo end",
+		      __func__);
+		return SLURM_ERROR;
+	}
+	return SLURM_SUCCESS;
+
+unpack_error:
+	return SLURM_ERROR;
 }
 
-extern int  switch_g_alloc_jobinfo(dynamic_plugin_data_t **jobinfo,
-				   uint32_t job_id, uint32_t step_id)
+extern int switch_g_build_stepinfo(dynamic_plugin_data_t **stepinfo,
+				   slurm_step_layout_t *step_layout,
+				   step_record_t *step_ptr)
 {
-	dynamic_plugin_data_t *jobinfo_ptr = NULL;
+	void **data = NULL;
+	uint32_t plugin_id = switch_context_default;
 
 	xassert(switch_context_cnt >= 0);
 
 	if (!switch_context_cnt)
 		return SLURM_SUCCESS;
 
-	jobinfo_ptr = _create_dynamic_plugin_data(switch_context_default);
-	*jobinfo    = jobinfo_ptr;
+	*stepinfo = _create_dynamic_plugin_data(plugin_id);
+	data = &(*stepinfo)->data;
 
-	return (*(ops[jobinfo_ptr->plugin_id].alloc_jobinfo))
-		((switch_jobinfo_t **)&jobinfo_ptr->data, job_id, step_id);
+	return (*(ops[plugin_id].build_stepinfo))((switch_stepinfo_t **) data,
+						  step_layout, step_ptr);
 }
 
-extern int switch_g_build_jobinfo(dynamic_plugin_data_t *jobinfo,
-				  slurm_step_layout_t *step_layout,
-				  step_record_t *step_ptr)
-{
-	void *data = NULL;
-	uint32_t plugin_id;
-
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
-	} else
-		plugin_id = switch_context_default;
-
-	return (*(ops[plugin_id].build_jobinfo))(data, step_layout, step_ptr);
-}
-
-extern int  switch_g_duplicate_jobinfo(dynamic_plugin_data_t *source,
-				       dynamic_plugin_data_t **dest)
+extern void switch_g_duplicate_stepinfo(dynamic_plugin_data_t *source,
+					dynamic_plugin_data_t **dest)
 {
 	dynamic_plugin_data_t *dest_ptr = NULL;
 	uint32_t plugin_id = source->plugin_id;
@@ -337,48 +362,52 @@ extern int  switch_g_duplicate_jobinfo(dynamic_plugin_data_t *source,
 	xassert(switch_context_cnt >= 0);
 
 	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
+		return;
 
 	dest_ptr = _create_dynamic_plugin_data(plugin_id);
 	*dest = dest_ptr;
 
-	return (*(ops[plugin_id].duplicate_jobinfo))(
-		source->data, (switch_jobinfo_t **)&dest_ptr->data);
+	(*(ops[plugin_id].duplicate_stepinfo))
+		(source->data, (switch_stepinfo_t **) &dest_ptr->data);
 }
 
-extern void switch_g_free_jobinfo(dynamic_plugin_data_t *jobinfo)
+extern void switch_g_free_stepinfo(dynamic_plugin_data_t *stepinfo)
 {
-	xassert(switch_context_cnt >= 0);
-
 	if (!switch_context_cnt)
 		return;
 
-	if (jobinfo) {
-		if (jobinfo->data)
-			(*(ops[jobinfo->plugin_id].free_jobinfo))
-				(jobinfo->data);
-		xfree(jobinfo);
+	if (stepinfo) {
+		if (stepinfo->data)
+			(*(ops[stepinfo->plugin_id].free_stepinfo))
+				(stepinfo->data);
+		xfree(stepinfo);
 	}
 }
 
-extern int switch_g_pack_jobinfo(dynamic_plugin_data_t *jobinfo, buf_t *buffer,
-				 uint16_t protocol_version)
+extern void switch_g_pack_stepinfo(dynamic_plugin_data_t *stepinfo,
+				   buf_t *buffer, uint16_t protocol_version)
 {
 	void *data = NULL;
-	uint32_t plugin_id;
+	uint32_t length_position = 0, start = 0, end = 0, plugin_id;
 
 	xassert(switch_context_cnt >= 0);
 
-	if (!switch_context_cnt) {
-		if (protocol_version <= SLURM_23_02_PROTOCOL_VERSION) {
-			pack32(SWITCH_PLUGIN_NONE, buffer);
-		}
-		return SLURM_SUCCESS;
+	if (protocol_version >= SLURM_24_11_PROTOCOL_VERSION) {
+		length_position = get_buf_offset(buffer);
+		pack32(0, buffer);
+		start = get_buf_offset(buffer);
 	}
 
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
+	if (!switch_context_cnt) {
+		/* Remove when 23.02 is no longer supported. */
+		if (protocol_version <= SLURM_23_02_PROTOCOL_VERSION)
+			pack32(SWITCH_PLUGIN_NONE, buffer);
+		return;
+	}
+
+	if (stepinfo) {
+		data = stepinfo->data;
+		plugin_id = stepinfo->plugin_id;
 	} else
 		plugin_id = switch_context_default;
 
@@ -387,50 +416,74 @@ extern int switch_g_pack_jobinfo(dynamic_plugin_data_t *jobinfo, buf_t *buffer,
 	} else {
 		error("%s: protocol_version %hu not supported",
 		      __func__, protocol_version);
-		return SLURM_ERROR;
+		return;
 	}
 
-	return (*(ops[plugin_id].pack_jobinfo))(data, buffer, protocol_version);
+	(*(ops[plugin_id].pack_stepinfo))(data, buffer, protocol_version);
+
+	if (protocol_version >= SLURM_24_11_PROTOCOL_VERSION) {
+		end = get_buf_offset(buffer);
+		set_buf_offset(buffer, length_position);
+		pack32(end - start, buffer);
+		set_buf_offset(buffer, end);
+	}
 }
 
-extern int switch_g_unpack_jobinfo(dynamic_plugin_data_t **jobinfo,
-				   buf_t *buffer, uint16_t protocol_version)
+extern int switch_g_unpack_stepinfo(dynamic_plugin_data_t **stepinfo,
+				    buf_t *buffer, uint16_t protocol_version)
 {
-	dynamic_plugin_data_t *jobinfo_ptr = NULL;
+	int i;
+	uint32_t length = 0, switch_stepinfo_end = 0, plugin_id;
+	dynamic_plugin_data_t *stepinfo_ptr = NULL;
 
 	xassert(switch_context_cnt >= 0);
 
-	if (!switch_context_cnt) {
+	if (protocol_version < SLURM_MIN_PROTOCOL_VERSION)
+		goto unpack_error;
+
+	if (protocol_version >= SLURM_24_11_PROTOCOL_VERSION) {
+		safe_unpack32(&length, buffer);
+		switch_stepinfo_end = get_buf_offset(buffer) + length;
+		if (!running_in_slurmstepd() || !length || !switch_context_cnt)
+			goto skip_buf;
+
+		if (remaining_buf(buffer) < length)
+			return SLURM_ERROR;
+	} else if (!switch_context_cnt) {
+		/* Remove when 23.02 is no longer supported. */
 		if (protocol_version <= SLURM_23_02_PROTOCOL_VERSION) {
-			uint32_t plugin_id;
 			safe_unpack32(&plugin_id, buffer);
-			*jobinfo = NULL;
+			*stepinfo = NULL;
 		}
 		return SLURM_SUCCESS;
 	}
 
-	jobinfo_ptr = xmalloc(sizeof(dynamic_plugin_data_t));
-	*jobinfo = jobinfo_ptr;
+	stepinfo_ptr = xmalloc(sizeof(dynamic_plugin_data_t));
+	*stepinfo = stepinfo_ptr;
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		int i;
-		uint32_t plugin_id;
-		safe_unpack32(&plugin_id, buffer);
-		for (i = 0; i < switch_context_cnt; i++) {
-			if (*(ops[i].plugin_id) == plugin_id) {
-				jobinfo_ptr->plugin_id = i;
-				break;
-			}
+	safe_unpack32(&plugin_id, buffer);
+	for (i = 0; i < switch_context_cnt; i++) {
+		if (*(ops[i].plugin_id) == plugin_id) {
+			stepinfo_ptr->plugin_id = i;
+			break;
 		}
-		if (i >= switch_context_cnt) {
-			error("we don't have switch plugin type %u", plugin_id);
-			goto unpack_error;
+	}
+
+	if (i >= switch_context_cnt) {
+		if (protocol_version >= SLURM_24_11_PROTOCOL_VERSION) {
+			/*
+			 * We were sent a plugin that we don't know how to
+			 * handle so skip it if possible.
+			 */
+			debug("we don't have switch plugin type %u", plugin_id);
+			goto skip_buf;
 		}
-	} else
+		error("we don't have switch plugin type %u", plugin_id);
 		goto unpack_error;
+	}
 
-	if  ((*(ops[jobinfo_ptr->plugin_id].unpack_jobinfo))
-	     ((switch_jobinfo_t **)&jobinfo_ptr->data, buffer,
+	if ((*(ops[stepinfo_ptr->plugin_id].unpack_stepinfo))
+	     ((switch_stepinfo_t **) &stepinfo_ptr->data, buffer,
 	      protocol_version))
 		goto unpack_error;
 
@@ -438,18 +491,25 @@ extern int switch_g_unpack_jobinfo(dynamic_plugin_data_t **jobinfo,
 	 * Free nodeinfo_ptr if it is different from local cluster as it is not
 	 * relevant to this cluster.
 	 */
-	if ((jobinfo_ptr->plugin_id != switch_context_default) &&
+	if ((stepinfo_ptr->plugin_id != switch_context_default) &&
 	    running_in_slurmctld()) {
-		switch_g_free_jobinfo(jobinfo_ptr);
-		*jobinfo = _create_dynamic_plugin_data(switch_context_default);
+		switch_g_free_stepinfo(stepinfo_ptr);
+		*stepinfo = _create_dynamic_plugin_data(switch_context_default);
 	}
 
+	return SLURM_SUCCESS;
 
+skip_buf:
+	if (length) {
+		debug("%s: skipping switch_stepinfo data (%u)",
+		      __func__, length);
+		set_buf_offset(buffer, switch_stepinfo_end);
+	}
 	return SLURM_SUCCESS;
 
 unpack_error:
-	switch_g_free_jobinfo(jobinfo_ptr);
-	*jobinfo = NULL;
+	switch_g_free_stepinfo(stepinfo_ptr);
+	*stepinfo = NULL;
 	error("%s: unpack error", __func__);
 	return SLURM_ERROR;
 }
@@ -474,25 +534,6 @@ extern int switch_g_job_init(stepd_step_rec_t *step)
 	return (*(ops[switch_context_default].job_init))(step);
 }
 
-extern int switch_g_job_fini(dynamic_plugin_data_t *jobinfo)
-{
-	void *data = NULL;
-	uint32_t plugin_id;
-
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
-	} else
-		plugin_id = switch_context_default;
-
-	return (*(ops[plugin_id].job_fini)) (data);
-}
-
 extern int switch_g_job_postfini(stepd_step_rec_t *step)
 {
 	xassert(switch_context_cnt >= 0);
@@ -503,7 +544,7 @@ extern int switch_g_job_postfini(stepd_step_rec_t *step)
 	return (*(ops[switch_context_default].job_postfini))(step);
 }
 
-extern int switch_g_job_attach(dynamic_plugin_data_t *jobinfo, char ***env,
+extern int switch_g_job_attach(dynamic_plugin_data_t *stepinfo, char ***env,
 			       uint32_t nodeid, uint32_t procid,
 			       uint32_t nnodes, uint32_t nprocs, uint32_t gid)
 {
@@ -515,9 +556,9 @@ extern int switch_g_job_attach(dynamic_plugin_data_t *jobinfo, char ***env,
 	if (!switch_context_cnt)
 		return SLURM_SUCCESS;
 
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
+	if (stepinfo) {
+		data = stepinfo->data;
+		plugin_id = stepinfo->plugin_id;
 	} else
 		plugin_id = switch_context_default;
 
@@ -525,7 +566,7 @@ extern int switch_g_job_attach(dynamic_plugin_data_t *jobinfo, char ***env,
 		(data, env, nodeid, procid, nnodes, nprocs, gid);
 }
 
-extern int switch_g_job_step_complete(dynamic_plugin_data_t *jobinfo,
+extern int switch_g_job_step_complete(dynamic_plugin_data_t *stepinfo,
 				      char *nodelist)
 {
 	void *data = NULL;
@@ -536,81 +577,70 @@ extern int switch_g_job_step_complete(dynamic_plugin_data_t *jobinfo,
 	if (!switch_context_cnt)
 		return SLURM_SUCCESS;
 
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
+	if (stepinfo) {
+		data = stepinfo->data;
+		plugin_id = stepinfo->plugin_id;
 	} else
 		plugin_id = switch_context_default;
 
 	return (*(ops[plugin_id].step_complete))(data, nodelist);
 }
 
-extern int switch_g_job_step_allocated(dynamic_plugin_data_t *jobinfo,
-				       char *nodelist)
-{
-	void *data = NULL;
-	uint32_t plugin_id;
-
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	if (jobinfo) {
-		data      = jobinfo->data;
-		plugin_id = jobinfo->plugin_id;
-	} else
-		plugin_id = switch_context_default;
-
-	return (*(ops[plugin_id].step_allocated))(data, nodelist);
-}
-
-extern int switch_g_job_step_pre_suspend(stepd_step_rec_t *step)
-{
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	return (*(ops[switch_context_default].job_step_pre_suspend))(step);
-}
-
-extern int switch_g_job_step_post_suspend(stepd_step_rec_t *step)
-{
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	return (*(ops[switch_context_default].job_step_post_suspend))(step);
-}
-
-extern int switch_g_job_step_pre_resume(stepd_step_rec_t *step)
-{
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	return (*(ops[switch_context_default].job_step_pre_resume))(step);
-}
-
-extern int switch_g_job_step_post_resume(stepd_step_rec_t *step)
-{
-	xassert(switch_context_cnt >= 0);
-
-	if (!switch_context_cnt)
-		return SLURM_SUCCESS;
-
-	return (*(ops[switch_context_default].job_step_post_resume))(step);
-}
-
-extern void switch_g_job_complete(uint32_t job_id)
+extern void switch_g_job_start(job_record_t *job_ptr)
 {
 	xassert(switch_context_cnt >= 0);
 
 	if (!switch_context_cnt)
 		return;
 
-	(*(ops[switch_context_default].job_complete))(job_id);
+	(*(ops[switch_context_default].job_start))(job_ptr);
 }
+
+extern void switch_g_job_complete(job_record_t *job_ptr)
+{
+	xassert(switch_context_cnt >= 0);
+
+	if (!switch_context_cnt)
+		return;
+
+	(*(ops[switch_context_default].job_complete))(job_ptr);
+}
+
+extern int switch_g_fs_init(stepd_step_rec_t *step)
+{
+	xassert(switch_context_cnt >= 0);
+
+	if (!switch_context_cnt)
+		return SLURM_SUCCESS;
+
+	return (*(ops[switch_context_default].fs_init))(step);
+}
+
+extern void switch_g_extern_stepinfo(void **stepinfo, job_record_t *job_ptr)
+{
+	switch_stepinfo_t *tmp = NULL;
+	dynamic_plugin_data_t *dest_ptr = NULL;
+
+	xassert(switch_context_cnt >= 0);
+
+	if (!switch_context_cnt)
+		return;
+
+	(*(ops[switch_context_default].extern_stepinfo))(&tmp, job_ptr);
+
+	if (tmp) {
+		dest_ptr = _create_dynamic_plugin_data(switch_context_default);
+		dest_ptr->data = tmp;
+		*stepinfo = dest_ptr;
+	}
+}
+
+extern void switch_g_extern_step_fini(uint32_t job_id)
+{
+	xassert(switch_context_cnt >= 0);
+
+	if (!switch_context_cnt)
+		return;
+
+	(*(ops[switch_context_default].extern_step_fini))(job_id);
+}
\ No newline at end of file
