@@ -34,6 +34,7 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+#include "as_mysql_cluster.h"
 #include "as_mysql_convert.h"
 #include "as_mysql_tres.h"
 #include "src/interfaces/jobacct_gather.h"
@@ -43,8 +44,9 @@
  * NOTE: 13 was the first version of 23.02.
  * NOTE: 14 was the first version of 23.11.
  * NOTE: 15 was the second version of 23.11.
+ * NOTE: 16 was the first version of 24.11.
  */
-#define CONVERT_VERSION 15
+#define CONVERT_VERSION 16
 
 #define MIN_CONVERT_VERSION 13
 
@@ -149,6 +151,28 @@ static int _set_db_curr_ver(mysql_conn_t *mysql_conn)
 	return rc;
 }
 
+extern int as_mysql_print_dbver(mysql_conn_t *mysql_conn)
+{
+	int rc = 0;
+
+	as_mysql_convert_possible(mysql_conn);
+
+	if (db_curr_ver > CONVERT_VERSION) {
+		printf("Slurm Database is somehow higher than expected '%u' but I only know as high as '%u'. Conversion needed.\n",
+		       db_curr_ver, CONVERT_VERSION);
+		rc = 1;
+	} else if (db_curr_ver != CONVERT_VERSION) {
+		printf("Slurm Database current version is '%u' needs to be at '%u'. Conversion needed.\n",
+		       db_curr_ver, CONVERT_VERSION);
+		rc = 1;
+	} else {
+		printf("Slurm Database already at the highest version '%u'. No conversion needed.\n",
+		       db_curr_ver);
+	}
+
+	return rc;
+}
+
 extern void as_mysql_convert_possible(mysql_conn_t *mysql_conn)
 {
 	(void) _set_db_curr_ver(mysql_conn);
@@ -386,6 +410,20 @@ static int _foreach_post_create(void *x, void *arg)
 	     SLURM_SUCCESS)
 		return rc;
 
+	if (db_curr_ver < 16) {
+		uint16_t id = as_mysql_cluster_get_unique_id(
+			mysql_conn, cluster_name);
+		char *query = xstrdup_printf(
+			"update %s set id=%u, mod_time=UNIX_TIMESTAMP() where name='%s'",
+			cluster_table, id, cluster_name);
+
+		info("Setting cluster id '%u' on cluster %s", id, cluster_name);
+
+		DB_DEBUG(DB_QUERY, mysql_conn->conn, "query\n%s", query);
+		rc = mysql_db_query(mysql_conn, query);
+		xfree(query);
+	}
+
 	return SLURM_SUCCESS;
 }
 
