@@ -96,7 +96,7 @@ static void *_cluster_rollup_usage(void *arg)
 	xassert(rollup_stats);
 
 	memset(&mysql_conn, 0, sizeof(mysql_conn_t));
-	mysql_conn.rollback = 1;
+	mysql_conn.flags |= DB_CONN_FLAG_ROLLBACK;
 	mysql_conn.conn = local_rollup->mysql_conn->conn;
 	slurm_mutex_init(&mysql_conn.lock);
 
@@ -414,24 +414,27 @@ end_it:
 static int _get_object_usage(mysql_conn_t *mysql_conn,
 			     slurmdbd_msg_type_t type, char *my_usage_table,
 			     char *cluster_name, char *id_str,
-			     time_t start, time_t end, List *usage_list)
+			     time_t start, time_t end, list_t **usage_list)
 {
 	char *tmp = NULL;
 	int i = 0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	char *query = NULL;
-	assoc_mgr_lock_t locks = { NO_LOCK, NO_LOCK, NO_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+	assoc_mgr_lock_t locks = {
+		.tres = READ_LOCK,
+	};
 
 	char *usage_req_inx[] = {
 		"t3.id_assoc",
+		"t1.id_alt",
 		"t1.id_tres",
 		"t1.time_start",
 		"t1.alloc_secs",
 	};
 	enum {
 		USAGE_ID,
+		USAGE_ID_ALT,
 		USAGE_TRES,
 		USAGE_START,
 		USAGE_ALLOC,
@@ -447,17 +450,18 @@ static int _get_object_usage(mysql_conn_t *mysql_conn,
 	}
 
 	switch (type) {
+	case DBD_GET_QOS_USAGE:
 	case DBD_GET_ASSOC_USAGE:
 		query = xstrdup_printf(
-			"select %s from \"%s_%s\" as t1, "
-			"\"%s_%s\" as t2, \"%s_%s\" as t3 "
-			"where (t1.time_start < %ld && t1.time_start >= %ld) "
-			"&& t1.id=t2.id_assoc && (%s) && "
-			"t2.lft between t3.lft and t3.rgt "
-			"order by t3.id_assoc, time_start;",
-			tmp, cluster_name, my_usage_table,
-			cluster_name, assoc_table, cluster_name, assoc_table,
-			end, start, id_str);
+		        "select %s from \"%s_%s\" as t1, "
+		        "\"%s_%s\" as t2, \"%s_%s\" as t3 "
+		        "where (t1.time_start < %ld && t1.time_start >= %ld) "
+		        "&& t1.id=t2.id_assoc && (%s) && "
+		        "t2.lineage like concat(t3.lineage, '%%') "
+		        "order by t3.id_assoc, time_start;",
+		        tmp, cluster_name, my_usage_table,
+		        cluster_name, assoc_table, cluster_name, assoc_table,
+		        end, start, id_str);
 		break;
 	case DBD_GET_WCKEY_USAGE:
 		query = xstrdup_printf(
@@ -501,6 +505,7 @@ static int _get_object_usage(mysql_conn_t *mysql_conn,
 		}
 
 		accounting_rec->id = slurm_atoul(row[USAGE_ID]);
+		accounting_rec->id_alt = slurm_atoul(row[USAGE_ID_ALT]);
 		accounting_rec->period_start = slurm_atoul(row[USAGE_START]);
 		accounting_rec->alloc_secs = slurm_atoull(row[USAGE_ALLOC]);
 
@@ -526,8 +531,9 @@ static int _get_cluster_usage(mysql_conn_t *mysql_conn, uid_t uid,
 	char *tmp = NULL;
 	char *my_usage_table = cluster_day_table;
 	char *query = NULL;
-	assoc_mgr_lock_t locks = { NO_LOCK, NO_LOCK, NO_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+	assoc_mgr_lock_t locks = {
+		.tres = READ_LOCK,
+	};
 	char *cluster_req_inx[] = {
 		"id_tres",
 		"alloc_secs",
@@ -624,15 +630,18 @@ static int _get_cluster_usage(mysql_conn_t *mysql_conn, uid_t uid,
    user or not.  The assoc_mgr locks should be unlocked before coming here.
 */
 extern int get_usage_for_list(mysql_conn_t *mysql_conn,
-			      slurmdbd_msg_type_t type, List object_list,
+			      slurmdbd_msg_type_t type, void *object_in,
 			      char *cluster_name, time_t start, time_t end)
 {
 	int rc = SLURM_SUCCESS;
 	char *my_usage_table = NULL;
-	List usage_list = NULL;
-	char *id_str = NULL, *name_char = NULL;
-	ListIterator itr = NULL, u_itr = NULL;
+	list_t *usage_list = NULL;
+	char *id_str = NULL, *name_char = NULL, *pos = NULL;
+	list_itr_t *itr = NULL, *u_itr = NULL;
 	void *object = NULL;
+	list_t *object_list = object_in;
+	usage_qos_query_t *qos_usage = object_in;
+	bool first = false;
 	slurmdb_assoc_rec_t *assoc = NULL;
 	slurmdb_wckey_rec_t *wckey = NULL;
 	slurmdb_accounting_rec_t *accounting_rec = NULL;
@@ -659,6 +668,39 @@ extern int get_usage_for_list(mysql_conn_t *mysql_conn,
 		list_iterator_destroy(itr);
 		my_usage_table = assoc_day_table;
 		break;
+	case DBD_GET_QOS_USAGE:
+		if (qos_usage->qos_list) {
+			first = true;
+			itr = list_iterator_create(qos_usage->qos_list);
+			while ((name_char = list_next(itr))) {
+				if (!first)
+					xstrfmtcatat(id_str, &pos, ",%s",
+						     name_char);
+				else {
+					xstrfmtcatat(id_str, &pos,
+						     "id_alt in (%s",
+						     name_char);
+					first = false;
+				}
+			}
+			list_iterator_destroy(itr);
+		}
+		my_usage_table = qos_day_table;
+		first = true;
+		itr = list_iterator_create(qos_usage->assoc_list);
+		while ((assoc = list_next(itr))) {
+			if (!first)
+				xstrfmtcat(id_str, ",%u", assoc->id);
+			else {
+				xstrfmtcat(id_str, "%st3.id_assoc in (%u",
+					   id_str ? ") && " : "",
+					   assoc->id);
+				first = false;
+			}
+		}
+		list_iterator_destroy(itr);
+		object_list = qos_usage->assoc_list;
+		break;
 	case DBD_GET_WCKEY_USAGE:
 		name_char = "id";
 		itr = list_iterator_create(object_list);
@@ -706,9 +748,10 @@ extern int get_usage_for_list(mysql_conn_t *mysql_conn,
 	while ((object = list_next(itr))) {
 		int found = 0;
 		int id = 0;
-		List acct_list = NULL;
+		list_t *acct_list = NULL;
 
 		switch (type) {
+		case DBD_GET_QOS_USAGE:
 		case DBD_GET_ASSOC_USAGE:
 			assoc = (slurmdb_assoc_rec_t *)object;
 			if (!assoc->accounting_list)
@@ -774,24 +817,49 @@ extern int as_mysql_get_usage(mysql_conn_t *mysql_conn, uid_t uid,
 	slurmdb_assoc_rec_t *slurmdb_assoc = in;
 	slurmdb_wckey_rec_t *slurmdb_wckey = in;
 	char *username = NULL;
-	List *my_list = NULL;
+	list_t **my_list = NULL;
 	char *cluster_name = NULL;
-	char *id_str = NULL;
+	char *id_str = NULL, *id_str_pos = NULL;
 
 	if (check_connection(mysql_conn) != SLURM_SUCCESS)
 		return ESLURM_DB_CONNECTION;
 
 	switch (type) {
+	case DBD_GET_QOS_USAGE:
 	case DBD_GET_ASSOC_USAGE:
 		if (!slurmdb_assoc->id) {
 			error("We need an id to set data for getting usage");
 			return SLURM_ERROR;
 		}
-		id_str = xstrdup_printf("t3.id_assoc=%u", slurmdb_assoc->id);
+		xstrfmtcatat(id_str, &id_str_pos, "t3.id_assoc=%u",
+			     slurmdb_assoc->id);
+		my_usage_table = assoc_day_table;
+		if (slurmdb_assoc->qos_list &&
+		    list_count(slurmdb_assoc->qos_list)) {
+			list_itr_t *itr = list_iterator_create(
+				slurmdb_assoc->qos_list);
+			char *qos_id;
+			bool first = true;
+			while ((qos_id = list_next(itr))) {
+				if (!first) {
+					xstrfmtcatat(id_str, &id_str_pos,
+						     ",%s", qos_id);
+				} else {
+					xstrfmtcatat(id_str, &id_str_pos,
+						     " && id_alt in (%s",
+						     qos_id);
+					first = false;
+				}
+			}
+			list_iterator_destroy(itr);
+			if (!first)
+				xstrcatat(id_str, &id_str_pos, ")");
+			my_usage_table = qos_day_table;
+			type = DBD_GET_QOS_USAGE;
+		}
 		cluster_name = slurmdb_assoc->cluster;
 		username = slurmdb_assoc->user;
 		my_list = &slurmdb_assoc->accounting_list;
-		my_usage_table = assoc_day_table;
 		break;
 	case DBD_GET_WCKEY_USAGE:
 		if (!slurmdb_wckey->id) {
@@ -824,7 +892,7 @@ extern int as_mysql_get_usage(mysql_conn_t *mysql_conn, uid_t uid,
 	if (slurm_conf.private_data & PRIVATE_DATA_USAGE) {
 		if (!(is_admin = is_user_min_admin_level(
 			      mysql_conn, uid, SLURMDB_ADMIN_OPERATOR))) {
-			ListIterator itr = NULL;
+			list_itr_t *itr = NULL;
 			slurmdb_coord_rec_t *coord = NULL;
 			slurmdb_user_rec_t user;
 			bool is_coord;
@@ -850,7 +918,13 @@ extern int as_mysql_get_usage(mysql_conn_t *mysql_conn, uid_t uid,
 				goto bad_user;
 			}
 
-			/* Existance of user.coord_accts is checked in
+			if (slurmdbd_conf->flags &
+			    DBD_CONF_FLAG_DISABLE_COORD_DBD) {
+				debug4("Coordinator privilege revoked with DisableCoordDBD.");
+				goto bad_user;
+			}
+
+			/* Existence of user.coord_accts is checked in
 			   is_user_any_coord.
 			*/
 			itr = list_iterator_create(user.coord_accts);
@@ -886,13 +960,13 @@ is_user:
 
 extern int as_mysql_roll_usage(mysql_conn_t *mysql_conn, time_t sent_start,
 			       time_t sent_end, uint16_t archive_data,
-			       List *rollup_stats_list_in)
+			       list_t **rollup_stats_list_in)
 {
 	int rc = SLURM_SUCCESS;
 	int rolledup = 0;
 	int roll_started = 0;
 	char *cluster_name = NULL;
-	ListIterator itr;
+	list_itr_t *itr;
 	pthread_mutex_t rolledup_lock = PTHREAD_MUTEX_INITIALIZER;
 	pthread_cond_t rolledup_cond;
 	//DEF_TIMERS;
@@ -944,7 +1018,7 @@ extern int as_mysql_roll_usage(mysql_conn_t *mysql_conn, time_t sent_start,
 		 * fashion buys a bunch on systems with lots
 		 * (millions) of jobs.
 		 */
-		slurm_thread_create_detached(NULL, _cluster_rollup_usage,
+		slurm_thread_create_detached(_cluster_rollup_usage,
 					     local_rollup);
 		roll_started++;
 	}
