@@ -2,7 +2,6 @@
  *  auth_slurm.c
  *****************************************************************************
  *  Copyright (C) SchedMD LLC.
- *  Written by Tim Wickberg <tim@schedmd.com>
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -146,7 +145,12 @@ extern int fini(void)
 	fini_run = true;
 
 	if (internal) {
-		fini_sack_conmgr();
+		/*
+		 * Do not attempt to remove /run/slurm/sack.socket.
+		 * If multiple daemons are co-located on this node, we may no
+		 * longer be the one that owns that socket, and removing it
+		 * would prevent the current owner from responding.
+		 */
 		fini_internal();
 	}
 
@@ -249,7 +253,6 @@ extern int auth_p_pack(auth_cred_t *cred, buf_t *buf,
 extern auth_cred_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 {
 	auth_cred_t *cred = NULL;
-	uint32_t uint32_tmp;
 
 	if (!buf) {
 		errno = ESLURM_AUTH_BADARG;
@@ -257,7 +260,7 @@ extern auth_cred_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 	}
 
 	cred = new_cred();
-	safe_unpackstr_xmalloc(&cred->token, &uint32_tmp, buf);
+	safe_unpackstr(&cred->token, buf);
 
 	return cred;
 
