@@ -5,8 +5,7 @@
  *  buffer plugins at the same time, so the state information is largely in the
  *  individual plugin and passed as a pointer argument to these functions.
  *****************************************************************************
- *  Copyright (C) 2014-2015 SchedMD LLC.
- *  Written by Morris Jette <jette@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -44,15 +43,15 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <poll.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <sys/mman.h>	/* memfd_create */
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
-#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
+#ifndef POLLRDHUP
 #define POLLRDHUP POLLHUP
-#include <signal.h>
 #endif
 
 #include "slurm/slurm.h"
@@ -65,7 +64,7 @@
 #include "src/common/pack.h"
 #include "src/common/parse_config.h"
 #include "src/common/run_command.h"
-#include "src/common/slurm_accounting_storage.h"
+#include "src/interfaces/accounting_storage.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/timers.h"
 #include "src/common/uid.h"
@@ -96,7 +95,7 @@ static uid_t *_parse_users(char *buf)
 		return user_array;
 	tmp = xstrdup(buf);
 	array_size = 1;
-	user_array = xmalloc(sizeof(uid_t) * array_size);
+	user_array = xcalloc(array_size, sizeof(uid_t));
 	tok = strtok_r(tmp, ",", &save_ptr);
 	while (tok) {
 		if ((uid_from_string(tok, user_array + inx) == -1) ||
@@ -140,9 +139,9 @@ static char *_print_users(uid_t *buf)
 /* Allocate burst buffer hash tables */
 extern void bb_alloc_cache(bb_state_t *state_ptr)
 {
-	state_ptr->bb_ahash = xmalloc(sizeof(bb_alloc_t *) * BB_HASH_SIZE);
-	state_ptr->bb_jhash = xmalloc(sizeof(bb_job_t *)   * BB_HASH_SIZE);
-	state_ptr->bb_uhash = xmalloc(sizeof(bb_user_t *)  * BB_HASH_SIZE);
+	state_ptr->bb_ahash = xcalloc(BB_HASH_SIZE, sizeof(bb_alloc_t *));
+	state_ptr->bb_jhash = xcalloc(BB_HASH_SIZE, sizeof(bb_job_t *));
+	state_ptr->bb_uhash = xcalloc(BB_HASH_SIZE, sizeof(bb_user_t *));
 }
 
 /* Clear all cached burst buffer records, freeing all memory. */
@@ -216,6 +215,7 @@ extern void bb_clear_config(bb_config_t *config_ptr, bool fini)
 	xfree(config_ptr->get_sys_state);
 	xfree(config_ptr->get_sys_status);
 	config_ptr->granularity = 1;
+	config_ptr->poll_interval = 0;
 	if (fini) {
 		for (i = 0; i < config_ptr->pool_cnt; i++)
 			xfree(config_ptr->pool_ptr[i].name);
@@ -351,6 +351,10 @@ char *_handle_replacement(job_record_t *job_ptr)
 				xstrfmtcat(replaced, "%u",
 					   job_ptr->array_task_id);
 				break;
+			case 'b':	/* '%b' => array task id modulo 10 */
+				xstrfmtcat(replaced, "%u",
+					   job_ptr->array_task_id % 10);
+				break;
 			case 'd':	/* '%d' => workdir */
 				xstrcat(replaced, job_ptr->details->work_dir);
 				break;
@@ -497,8 +501,8 @@ extern void bb_set_tres_pos(bb_state_t *state_ptr)
 	inx = assoc_mgr_find_tres_pos(&tres_rec, false);
 	state_ptr->tres_pos = inx;
 	if (inx == -1) {
-		debug3("%s: Tres %s not found by assoc_mgr",
-		       __func__, state_ptr->name);
+		debug3("Tres %s not found by assoc_mgr",
+		       state_ptr->name);
 	} else {
 		state_ptr->tres_id  = assoc_mgr_tres_array[inx]->id;
 	}
@@ -524,6 +528,7 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 		{"GetSysStatus", S_P_STRING},
 		{"Granularity", S_P_STRING},
 		{"OtherTimeout", S_P_UINT32},
+		{"PollInterval", S_P_UINT32},
 		{"Pools", S_P_STRING},
 		{"StageInTimeout", S_P_UINT32},
 		{"StageOutTimeout", S_P_UINT32},
@@ -548,6 +553,7 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 	/* Set default configuration */
 	bb_clear_config(&state_ptr->bb_config, false);
 	state_ptr->bb_config.flags |= BB_FLAG_DISABLE_PERSISTENT;
+	state_ptr->bb_config.poll_interval = DEFAULT_BB_POLL_INTERVAL;
 	state_ptr->bb_config.other_timeout = DEFAULT_OTHER_TIMEOUT;
 	state_ptr->bb_config.stage_in_timeout = DEFAULT_STATE_IN_TIMEOUT;
 	state_ptr->bb_config.stage_out_timeout = DEFAULT_STATE_OUT_TIMEOUT;
@@ -566,8 +572,8 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 		bb_conf = get_extra_conf_path(new_path);
 		fd = open(bb_conf, 0);
 		if (fd < 0) {
-			info("%s: Unable to find configuration file %s or "
-			     "burst_buffer.conf", __func__, new_path);
+			info("Unable to find configuration file %s or "
+			     "burst_buffer.conf", new_path);
 			xfree(bb_conf);
 			xfree(new_path);
 			return;
@@ -577,7 +583,7 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 	}
 
 	bb_hashtbl = s_p_hashtbl_create(bb_options);
-	if (s_p_parse_file(bb_hashtbl, NULL, bb_conf, false, NULL)
+	if (s_p_parse_file(bb_hashtbl, NULL, bb_conf, 0, NULL)
 	    == SLURM_ERROR) {
 		fatal("%s: something wrong with opening/reading %s: %m",
 		      __func__, bb_conf);
@@ -652,6 +658,8 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 		xfree(tmp);
 	}
 
+	(void) s_p_get_uint32(&state_ptr->bb_config.poll_interval,
+			     "PollInterval", bb_hashtbl);
 	(void) s_p_get_uint32(&state_ptr->bb_config.other_timeout,
 			     "OtherTimeout", bb_hashtbl);
 	(void) s_p_get_uint32(&state_ptr->bb_config.stage_in_timeout,
@@ -674,47 +682,49 @@ extern void bb_load_config(bb_state_t *state_ptr, char *plugin_type)
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_BURST_BUF) {
 		value = _print_users(state_ptr->bb_config.allow_users);
-		info("%s: AllowUsers:%s",  __func__, value);
+		info("AllowUsers:%s",  value);
 		xfree(value);
-		info("%s: CreateBuffer:%s",  __func__,
+		info("CreateBuffer:%s",
 		     state_ptr->bb_config.create_buffer);
-		info("%s: DefaultPool:%s",  __func__,
+		info("DefaultPool:%s",
 		     state_ptr->bb_config.default_pool);
 		value = _print_users(state_ptr->bb_config.deny_users);
-		info("%s: DenyUsers:%s",  __func__, value);
+		info("DenyUsers:%s",  value);
 		xfree(value);
-		info("%s: DestroyBuffer:%s",  __func__,
+		info("DestroyBuffer:%s",
 		     state_ptr->bb_config.destroy_buffer);
-		info("%s: Directive:%s",
-		     __func__, state_ptr->bb_config.directive_str);
-		info("%s: Flags:%s",
-		     __func__, slurm_bb_flags2str(state_ptr->bb_config.flags));
-		info("%s: GetSysState:%s",  __func__,
+		info("Directive:%s",
+		     state_ptr->bb_config.directive_str);
+		info("Flags:%s",
+		     slurm_bb_flags2str(state_ptr->bb_config.flags));
+		info("GetSysState:%s",
 		     state_ptr->bb_config.get_sys_state);
-		info("%s: GetSysStatus:%s",  __func__,
+		info("GetSysStatus:%s",
 		     state_ptr->bb_config.get_sys_status);
-		info("%s: Granularity:%"PRIu64"",  __func__,
+		info("Granularity:%"PRIu64"",
 		     state_ptr->bb_config.granularity);
 		for (i = 0; i < state_ptr->bb_config.pool_cnt; i++) {
-			info("%s: Pool[%d]:%s:%"PRIu64"", __func__, i,
+			info("Pool[%d]:%s:%"PRIu64"", i,
 			     state_ptr->bb_config.pool_ptr[i].name,
 			     state_ptr->bb_config.pool_ptr[i].total_space);
 		}
-		info("%s: OtherTimeout:%u", __func__,
+		info("PollInterval:%u",
+		     state_ptr->bb_config.poll_interval);
+		info("OtherTimeout:%u",
 		     state_ptr->bb_config.other_timeout);
-		info("%s: StageInTimeout:%u", __func__,
+		info("StageInTimeout:%u",
 		     state_ptr->bb_config.stage_in_timeout);
-		info("%s: StageOutTimeout:%u", __func__,
+		info("StageOutTimeout:%u",
 		     state_ptr->bb_config.stage_out_timeout);
-		info("%s: StartStageIn:%s",  __func__,
+		info("StartStageIn:%s",
 		     state_ptr->bb_config.start_stage_in);
-		info("%s: StartStageOut:%s",  __func__,
+		info("StartStageOut:%s",
 		     state_ptr->bb_config.start_stage_out);
-		info("%s: StopStageIn:%s",  __func__,
+		info("StopStageIn:%s",
 		     state_ptr->bb_config.stop_stage_in);
-		info("%s: StopStageOut:%s",  __func__,
+		info("StopStageOut:%s",
 		     state_ptr->bb_config.stop_stage_out);
-		info("%s: ValidateTimeout:%u", __func__,
+		info("ValidateTimeout:%u",
 		     state_ptr->bb_config.validate_timeout);
 	}
 }
@@ -765,6 +775,18 @@ static void _pack_alloc(struct bb_alloc *bb_alloc, buf_t *buffer,
 	}
 }
 
+/* Return true if hetjob separator in the script */
+static bool _hetjob_check(char *tok)
+{
+	if (xstrncmp(tok + 1, "SLURM",  5) &&
+	    xstrncmp(tok + 1, "SBATCH", 6))
+		return false;
+	if (!xstrstr(tok + 6, "packjob") &&
+	    !xstrstr(tok + 6, "hetjob"))
+		return false;
+	return true;
+}
+
 /* Pack individual burst buffer records into a buffer */
 extern int bb_pack_bufs(uid_t uid, bb_state_t *state_ptr, buf_t *buffer,
 			uint16_t protocol_version)
@@ -807,7 +829,37 @@ extern void bb_pack_state(bb_state_t *state_ptr, buf_t *buffer,
 	int i;
 
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_24_05_PROTOCOL_VERSION) {
+		packstr(config_ptr->allow_users_str, buffer);
+		packstr(config_ptr->create_buffer,   buffer);
+		packstr(config_ptr->default_pool,    buffer);
+		packstr(config_ptr->deny_users_str,  buffer);
+		packstr(config_ptr->destroy_buffer,  buffer);
+		pack32(config_ptr->flags,            buffer);
+		packstr(config_ptr->get_sys_state,   buffer);
+		packstr(config_ptr->get_sys_status,   buffer);
+		pack64(config_ptr->granularity,      buffer);
+		pack32(config_ptr->pool_cnt,         buffer);
+		for (i = 0; i < config_ptr->pool_cnt; i++) {
+			packstr(config_ptr->pool_ptr[i].name, buffer);
+			pack64(config_ptr->pool_ptr[i].total_space, buffer);
+			pack64(config_ptr->pool_ptr[i].granularity, buffer);
+			pack64(config_ptr->pool_ptr[i].unfree_space, buffer);
+			pack64(config_ptr->pool_ptr[i].used_space, buffer);
+		}
+		pack32(config_ptr->poll_interval, buffer);
+		pack32(config_ptr->other_timeout,    buffer);
+		packstr(config_ptr->start_stage_in,  buffer);
+		packstr(config_ptr->start_stage_out, buffer);
+		packstr(config_ptr->stop_stage_in,   buffer);
+		packstr(config_ptr->stop_stage_out,  buffer);
+		pack32(config_ptr->stage_in_timeout, buffer);
+		pack32(config_ptr->stage_out_timeout,buffer);
+		pack64(state_ptr->total_space,       buffer);
+		pack64(state_ptr->unfree_space,      buffer);
+		pack64(state_ptr->used_space,        buffer);
+		pack32(config_ptr->validate_timeout, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(config_ptr->allow_users_str, buffer);
 		packstr(config_ptr->create_buffer,   buffer);
 		packstr(config_ptr->default_pool,    buffer);
@@ -1151,6 +1203,7 @@ extern bb_alloc_t *bb_alloc_job_rec(bb_state_t *state_ptr,
 	bb_alloc->state_time = time(NULL);
 	bb_alloc->seen_time = time(NULL);
 	bb_alloc->user_id = job_ptr->user_id;
+	bb_alloc->group_id = job_ptr->group_id;
 
 	return bb_alloc;
 }
@@ -1181,6 +1234,52 @@ extern int bb_build_bb_script(job_record_t *job_ptr, char *script_file)
 	return rc;
 }
 
+extern char *bb_common_build_het_job_script(char *script,
+					    uint32_t het_job_offset,
+					    bool (*is_directive) (char *tok))
+{
+	char *result = NULL, *tmp = NULL;
+	char *tok, *save_ptr = NULL;
+	bool fini = false;
+	int cur_offset = 0;
+
+	tmp = xstrdup(script);
+	tok = strtok_r(tmp, "\n", &save_ptr);
+	while (tok) {
+		if (!result) {
+			xstrfmtcat(result, "%s\n", tok);
+		} else if (tok[0] != '#') {
+			fini = true;
+		} else if (_hetjob_check(tok)) {
+			cur_offset++;
+			if (cur_offset > het_job_offset)
+				fini = true;
+		} else if (cur_offset == het_job_offset) {
+			xstrfmtcat(result, "%s\n", tok);
+		}
+		if (fini)
+			break;
+		tok = strtok_r(NULL, "\n", &save_ptr);
+	}
+
+	if (het_job_offset == 0) {
+		while (tok) {
+			char *sep = "";
+			if (is_directive(tok)) {
+				sep = "#EXCLUDED ";
+				tok++;
+			}
+			xstrfmtcat(result, "%s%s\n", sep, tok);
+			tok = strtok_r(NULL, "\n", &save_ptr);
+		}
+	} else if (result) {
+		xstrcat(result, "exit 0\n");
+	}
+	xfree(tmp);
+
+	return result;
+}
+
 /* Free memory associated with allocated bb record, caller is responsible for
  * maintaining linked list */
 extern void bb_free_alloc_buf(bb_alloc_t *bb_alloc)
@@ -1418,8 +1517,8 @@ extern void bb_limit_rem(uint32_t user_id, uint64_t bb_size, char *pool,
 			 * after making a claim against resources, but before
 			 * the buffer actually gets created.
 			 */
-			debug2("%s: unfree_space underflow (%"PRIu64" < %"PRIu64")",
-			        __func__, state_ptr->unfree_space, bb_size);
+			debug2("unfree_space underflow (%"PRIu64" < %"PRIu64")",
+			        state_ptr->unfree_space, bb_size);
 			state_ptr->unfree_space = 0;
 		}
 	} else {
@@ -1442,8 +1541,8 @@ extern void bb_limit_rem(uint32_t user_id, uint64_t bb_size, char *pool,
 				 * state after making a claim against resources,
 				 * but before the buffer actually gets created.
 				 */
-				debug2("%s: unfree_space underflow for pool %s",
-				       __func__, pool);
+				debug2("unfree_space underflow for pool %s",
+				       pool);
 				pool_ptr->unfree_space = 0;
 			}
 			break;
@@ -1633,7 +1732,7 @@ static void _rm_active_job_bb(char *resv_name, char **pool_name,
 			      int64_t *resv_space, int ds_len,
 			      bb_state_t *bb_state)
 {
-	ListIterator job_iterator;
+	list_itr_t *job_iterator;
 	job_record_t *job_ptr;
 	bb_job_t *bb_job;
 	int i;
@@ -1680,8 +1779,8 @@ extern int bb_test_size_limit(job_record_t *job_ptr, bb_job_t *bb_job,
 	int i, j, k, rc = BB_CAN_START_NOW;
 	bool avail_ok, do_preempt, preempt_ok;
 	time_t now = time(NULL);
-	List preempt_list = NULL;
-	ListIterator preempt_iter;
+	list_t *preempt_list = NULL;
+	list_itr_t *preempt_iter;
 	bb_state_t bb_state = *bb_state_ptr;
 
 	xassert(bb_job);
@@ -1944,7 +2043,7 @@ extern void bb_update_system_comment(job_record_t *job_ptr, char *operation,
 		slurmdb_job_cond_t job_cond;
 		slurmdb_job_rec_t job_rec;
 		slurm_selected_step_t selected_step;
-		List ret_list;
+		list_t *ret_list;
 
 		memset(&job_cond, 0, sizeof(slurmdb_job_cond_t));
 		memset(&job_rec, 0, sizeof(slurmdb_job_rec_t));
@@ -1995,7 +2094,7 @@ extern bool bb_valid_pool_test(bb_state_t *state_ptr, char *pool_name)
 		if (!xstrcmp(pool_name, pool_ptr->name))
 			return true;
 	}
-	info("%s: Invalid pool requested (%s)", __func__, pool_name);
+	info("Invalid pool requested (%s)", pool_name);
 
 	return false;
 }
@@ -2087,7 +2186,7 @@ extern int bb_write_nid_file(char *file_name, char *node_list,
 
 	xassert(file_name);
 	if (node_list && node_list[0]) {
-		hostlist_t hl = hostlist_create(node_list);
+		hostlist_t *hl = hostlist_create(node_list);
 		while ((tok = hostlist_shift(hl))) {
 			xstrfmtcat(buf, "%s\n", tok);
 			free(tok);
@@ -2103,52 +2202,3 @@ extern int bb_write_nid_file(char *file_name, char *node_list,
 	return rc;
 #endif
 }
-
-extern void bb_write_state_file(char* old_file, char *reg_file, char *new_file,
-				const char *plugin, buf_t *buffer,
-				int buffer_size, time_t save_time,
-				time_t *last_save_time)
-{
-	int state_fd, error_code = 0;
-
-	state_fd = creat(new_file, 0600);
-	if (state_fd < 0) {
-		error("Can't save state, error creating file %s, %m",
-		      new_file);
-		error_code = errno;
-	} else {
-		int pos = 0, nwrite = get_buf_offset(buffer), amount, rc;
-		char *data = (char *)get_buf_data(buffer);
-		buffer_size = MAX(nwrite, buffer_size);
-		while (nwrite > 0) {
-			amount = write(state_fd, &data[pos], nwrite);
-			if ((amount < 0) && (errno != EINTR)) {
-				error("Error writing file %s, %m", new_file);
-				break;
-			}
-			nwrite -= amount;
-			pos    += amount;
-		}
-
-		rc = fsync_and_close(state_fd, plugin);
-		if (rc && !error_code)
-			error_code = rc;
-	}
-	if (error_code)
-		(void) unlink(new_file);
-	else {
-		/* file shuffle */
-		*last_save_time = save_time;
-		(void) unlink(old_file);
-		if (link(reg_file, old_file)) {
-			debug4("unable to create link for %s -> %s: %m",
-			       reg_file, old_file);
-		}
-		(void) unlink(reg_file);
-		if (link(new_file, reg_file)) {
-			debug4("unable to create link for %s -> %s: %m",
-			       new_file, reg_file);
-		}
-		(void) unlink(new_file);
-	}
-}
