@@ -49,6 +49,9 @@
 #define REF_PATH OPENAPI_PATH_REL OPENAPI_SCHEMAS_PATH
 #define TYPE_PREFIX "DATA_PARSER_"
 #define KEY_PREFIX XSTRINGIFY(DATA_VERSION) "_"
+#define OPENAPI_REF_PLACEHOLDER ((void *) 0xfa0b901301120040)
+#define IS_FLAG_BIT_DEPRECATED(bit) (bit->deprecated)
+#define IS_PARSER_DEPRECATED(parser) (parser->deprecated)
 
 typedef struct {
 	int magic; /* MAGIC_SPEC_ARGS */
@@ -61,43 +64,15 @@ typedef struct {
 	data_t *spec;
 	data_t *path_params; /* dict of each path param */
 	data_t *params; /* current parameters target */
-	int *references; /* references[i] = count(parsers[i]) */
 	bool disable_refs;
 } spec_args_t;
 
-#define MAGIC_REFS_PTR 0xaa910e8b
-
-typedef struct {
-	int magic; /* MAGIC_REFS_PTR */
-	int *references; /* references[i] = count(parsers[i]) */
-} refs_ptr_t;
-
+static void _add_parser(const parser_t *parser, spec_args_t *sargs);
 static void _replace_refs(data_t *data, spec_args_t *sargs);
-static void _count_refs(data_t *data, spec_args_t *sargs);
 extern void _set_ref(data_t *obj, const parser_t *parent,
 		     const parser_t *parser, spec_args_t *sargs);
 static data_t *_resolve_parser_key(const parser_t *parser, data_t *dst);
 
-static uint32_t _resolve_parser_index(const parser_t *parser,
-				      spec_args_t *sargs)
-{
-	for (int i = 0; i < sargs->parser_count; i++)
-		if (parser->type == sargs->parsers[i].type)
-			return i;
-
-	xassert(false);
-	return NO_VAL;
-}
-
-static const parser_t *_resolve_parser(const char *type, spec_args_t *sargs)
-{
-	for (int i = 0; i < sargs->parser_count; i++)
-		if (!xstrcmp(sargs->parsers[i].type_string, type))
-			return &sargs->parsers[i];
-
-	return NULL;
-}
-
 static char *_get_parser_key(const parser_t *parser)
 {
 	char *stype;
@@ -126,15 +101,8 @@ static char *_get_parser_path(const parser_t *parser)
 	return path;
 }
 
-/*
- * Populate OpenAPI specification field
- * IN obj - data_t ptr to specific field in OpenAPI schema
- * IN format - OpenAPI format to use
- * IN desc - Description of field to use
- * RET ptr to "items" for ARRAY or "properties" for OBJECT or NULL
- */
-static data_t *_set_openapi_props(data_t *obj, openapi_type_format_t format,
-				  const char *desc)
+extern data_t *set_openapi_props(data_t *obj, openapi_type_format_t format,
+				 const char *desc)
 {
 	data_t *dtype;
 	const char *format_str;
@@ -172,31 +140,14 @@ static data_t *_set_openapi_props(data_t *obj, openapi_type_format_t format,
 	return NULL;
 }
 
-static bool _should_be_ref(const parser_t *parser, spec_args_t *sargs)
+static bool _should_be_ref(const parser_t *parser)
 {
-	uint32_t parser_index;
-
-	if (sargs->disable_refs)
-		return false;
-
-	parser_index = _resolve_parser_index(parser, sargs);
-
-	/* parser with single reference doesn't need to be a $ref */
-	if (parser_index != NO_VAL) {
-		debug4("%s: %s references=%u",
-		       __func__, parser->type_string,
-		       sargs->references[parser_index]);
-
-		if (sargs->references[parser_index] <= 1)
-			return false;
-	}
-
 	if ((parser->obj_openapi == OPENAPI_FORMAT_OBJECT) ||
 	    (parser->obj_openapi == OPENAPI_FORMAT_ARRAY))
 		return true;
 
 	if (parser->array_type || parser->pointer_type || parser->list_type ||
-	    parser->fields)
+	    parser->fields || parser->alias_type)
 		return true;
 
 	return false;
@@ -209,13 +160,9 @@ static void _add_eflags(data_t *props, const parser_t *parser,
 
 	for (int i = 0; i < parser->flag_bit_array_count; i++) {
 		const flag_bit_t *bit = &parser->flag_bit_array[i];
-		data_t *dchild;
+		data_t *dchild = data_key_set(props, bit->name);
 
-		if (bit->hidden)
-			continue;
-
-		dchild = data_key_set(props, bit->name);
-		_set_openapi_props(dchild, OPENAPI_FORMAT_BOOL, NULL);
+		set_openapi_props(dchild, OPENAPI_FORMAT_BOOL, NULL);
 	}
 }
 
@@ -259,13 +206,14 @@ static void _add_param_flag_enum(data_t *param, const parser_t *parser)
  * IN obj - data_t ptr to specific field in OpenAPI schema
  * IN parser - populate field with info from parser
  * IN description - description from parent pointer parser or NULL
- * IN deprecated - parser marked as deprecated
+ *
+ * If parser is an ARRAY or OBJECT, the openapi_spec() function will be called
+ * from the parser to populate the child fields.
  *
  * RET ptr to "items" for ARRAY or "properties" for OBJECT or NULL
  */
 static data_t *_set_openapi_parse(data_t *obj, const parser_t *parser,
-				  spec_args_t *sargs, const char *desc,
-				  bool deprecated)
+				  spec_args_t *sargs, const char *desc)
 {
 	data_t *props;
 	openapi_type_format_t format;
@@ -274,11 +222,32 @@ static data_t *_set_openapi_parse(data_t *obj, const parser_t *parser,
 	xassert(sargs->magic == MAGIC_SPEC_ARGS);
 	xassert(sargs->args->magic == MAGIC_ARGS);
 	xassert(parser->model != PARSER_MODEL_ARRAY_SKIP_FIELD);
-	xassert(!parser->pointer_type);
-	xassert(parser->model != PARSER_MODEL_ARRAY_LINKED_FIELD);
-	xassert(parser->model !=
-		PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD);
-	xassert(parser->model != PARSER_MODEL_ARRAY_REMOVED_FIELD);
+
+	if ((parser->model ==
+	     PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD) ||
+	    (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD)) {
+		_set_ref(obj, parser, find_parser_by_type(parser->type), sargs);
+		return NULL;
+	} else if (parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD) {
+		/* find all parsers that should be references */
+		_set_ref(obj, parser, find_parser_by_type(parser->type), sargs);
+		return NULL;
+	} else if (parser->pointer_type || parser->alias_type) {
+		_set_ref(obj, parser, parser, sargs);
+		return NULL;
+	}
+
+	/* parser explicitly overrides the specification */
+	if (parser->openapi_spec) {
+		parser->openapi_spec(parser, sargs->args, sargs->spec, obj);
+
+		/* the resultant type must match the obj_openapi type */
+		xassert(is_complex_mode(sargs->args) ||
+			!xstrcmp(data_get_string(data_key_get(obj, "type")),
+				 openapi_type_format_to_type_string(
+					 parser->obj_openapi)));
+		return NULL;
+	}
 
 	if (parser->array_type || parser->list_type ||
 	    (parser->flag_bit_array && !parser->single_flag))
@@ -293,10 +262,10 @@ static data_t *_set_openapi_parse(data_t *obj, const parser_t *parser,
 	xassert(format > OPENAPI_FORMAT_INVALID);
 	xassert(format < OPENAPI_FORMAT_MAX);
 
-	if (parser->obj_desc && !desc)
+	if (parser->obj_desc)
 		desc = parser->obj_desc;
 
-	if ((props = _set_openapi_props(obj, format, desc))) {
+	if ((props = set_openapi_props(obj, format, desc))) {
 		if (parser->array_type) {
 			_set_ref(props, parser,
 				 find_parser_by_type(parser->array_type),
@@ -313,100 +282,71 @@ static data_t *_set_openapi_parse(data_t *obj, const parser_t *parser,
 			for (int i = 0; i < parser->field_count; i++)
 				_add_field(obj, required, parser,
 					   &parser->fields[i], sargs);
-		} else if (parser->model == PARSER_MODEL_REMOVED) {
-			/* do nothing */
 		} else if (!is_complex_mode(sargs->args)) {
 			fatal("%s: parser %s need to provide openapi specification, array type or pointer type",
 			      __func__, parser->type_string);
 		}
 	}
 
-	if (deprecated)
-		data_set_bool(data_key_set(obj, "deprecated"), true);
-
 	return props;
 }
 
+extern void set_openapi_parse_ref(data_t *obj, const parser_t *parser,
+				  data_t *spec, args_t *args)
+{
+	spec_args_t sargs = {
+		.magic = MAGIC_SPEC_ARGS,
+		.args = args,
+		.spec = spec,
+	};
+
+	xassert(parser->magic == MAGIC_PARSER);
+	xassert(args->magic == MAGIC_ARGS);
+
+	sargs.schemas = data_resolve_dict_path(spec, OPENAPI_SCHEMAS_PATH);
+
+	_set_ref(obj, NULL, parser, &sargs);
+}
+
 extern void _set_ref(data_t *obj, const parser_t *parent,
 		     const parser_t *parser, spec_args_t *sargs)
 {
-	char *str, *key;
+	char *str;
 	const char *desc = NULL;
-	bool deprecated = (parent && parent->deprecated);
+
+	if (parser->obj_desc)
+		desc = parser->obj_desc;
+	else if (parent && parent->obj_desc)
+		desc = parent->obj_desc;
 
 	xassert(sargs->magic == MAGIC_SPEC_ARGS);
 	xassert(sargs->args->magic == MAGIC_ARGS);
 
-	while (true) {
-		if (desc)
-			/* do nothing */;
-		else if (parent && parent->obj_desc)
-			desc = parent->obj_desc;
-		else if (parser->obj_desc)
+	while (parser->pointer_type || parser->alias_type) {
+		/* Not possible to use unalias_parser() here */
+
+		if (parser->obj_desc)
 			desc = parser->obj_desc;
 
-		/* All children are deprecated once the parent is */
-		if (parser->deprecated)
-			deprecated = true;
-
-		if (parser->model == PARSER_MODEL_REMOVED) {
-			if (is_complex_mode(sargs->args))
-				return;
-			break;
-		}
-
-		if ((parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD) ||
-		    (parser->model ==
-		     PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD) ||
-		    (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD)) {
-			/* resolve to linked parser */
-			parent = parser;
-			parser = find_parser_by_type(parser->type);
-
-			continue;
-		}
-
-		if (parser->pointer_type) {
+		if (parser->pointer_type)
 			parser = find_parser_by_type(parser->pointer_type);
-			continue;
-		}
-
-		break;
+		if (parser->alias_type)
+			parser = find_parser_by_type(parser->alias_type);
 	}
 
-	if (!_should_be_ref(parser, sargs)) {
-		_set_openapi_parse(obj, parser, sargs, desc, deprecated);
+	if (sargs->disable_refs || !_should_be_ref(parser)) {
+		_set_openapi_parse(obj, parser, sargs, desc);
 		return;
 	}
 
-	if (data_get_type(obj) == DATA_TYPE_NULL)
-		data_set_dict(obj);
-
-	xassert(data_get_type(obj) == DATA_TYPE_DICT);
-
+	data_set_dict(obj);
 	str = _get_parser_path(parser);
 	data_set_string_own(data_key_set(obj, "$ref"), str);
 
-	if (desc && !data_key_get(obj, "description"))
+	if (desc)
 		data_set_string(data_key_set(obj, "description"), desc);
 
-	if (deprecated)
-		data_set_bool(data_key_set(obj, "deprecated"), true);
-
-	/* Add schema for $ref target */
-
-	key = _get_parser_key(parser);
-	obj = data_key_set(sargs->schemas, key);
-
-	if (data_get_type(obj) == DATA_TYPE_NULL) {
-		debug4("%s: adding schema %s", __func__, key);
-		_set_openapi_parse(data_set_dict(obj), parser, sargs, NULL,
-				   parser->deprecated);
-	} else {
-		debug4("%s: skip adding duplicate schema %s", __func__, key);
-	}
-
-	xfree(key);
+	_add_parser(parser, sargs);
 }
 
 static data_t *_resolve_parser_key(const parser_t *parser, data_t *dst)
@@ -461,6 +401,39 @@ static data_t *_resolve_parser_key(const parser_t *parser, data_t *dst)
 	return dst;
 }
 
+static void _add_parser(const parser_t *parser, spec_args_t *sargs)
+{
+	data_t *obj;
+	char *key;
+
+	xassert(sargs->magic == MAGIC_SPEC_ARGS);
+	xassert(sargs->args->magic == MAGIC_ARGS);
+
+	if (!_should_be_ref(parser)) {
+	       debug3("%s: skip adding %s as simple type=%s format=%s",
+		      __func__, parser->type_string,
+		      openapi_type_format_to_type_string(
+			      parser->obj_openapi),
+		      openapi_type_format_to_format_string(
+			      parser->obj_openapi));
+	       return;
+	}
+
+	key = _get_parser_key(parser);
+	obj = data_key_set(sargs->schemas, key);
+
+	if (data_get_type(obj) != DATA_TYPE_NULL) {
+		debug3("%s: skip adding duplicate schema %s",
+		      __func__, key);
+		xfree(key);
+		return;
+	}
+	xfree(key);
+
+	data_set_dict(obj);
+	_set_openapi_parse(obj, parser, sargs, NULL);
+}
+
 static data_for_each_cmd_t _convert_list_entry(data_t *data, void *arg)
 {
 	spec_args_t *sargs = arg;
@@ -483,6 +456,33 @@ static data_for_each_cmd_t _convert_dict_entry(const char *key, data_t *data,
 	xassert(sargs->magic == MAGIC_SPEC_ARGS);
 	xassert(sargs->args->magic == MAGIC_ARGS);
 
+	if (!xstrcmp(key, "$ref") &&
+	    (data_get_type(data) == DATA_TYPE_STRING) &&
+	    !xstrncmp(data_get_string(data), TYPE_PREFIX,
+		      strlen(TYPE_PREFIX))) {
+		const parser_t *parser = NULL;
+		char *str;
+
+		for (int i = 0; i < sargs->parser_count; i++) {
+			if (!xstrcmp(sargs->parsers[i].type_string,
+				     data_get_string(data))) {
+				parser = &sargs->parsers[i];
+				break;
+			}
+		}
+
+		if (!parser) {
+			debug("%s: skipping unknown %s",
+			      __func__, data_get_string(data));
+			data_set_null(data);
+			return DATA_FOR_EACH_CONT;
+		}
+
+		str = _get_parser_path(parser);
+		data_set_string_own(data, str);
+		_add_parser(parser, sargs);
+	}
+
 	if ((data_get_type(data) == DATA_TYPE_LIST) ||
 	    (data_get_type(data) == DATA_TYPE_DICT))
 		_replace_refs(data, sargs);
@@ -494,107 +494,6 @@ static data_for_each_cmd_t _convert_dict_entry(const char *key, data_t *data,
  * Find every $ref = DATA_PARSER_* and add correct path
  */
 static void _replace_refs(data_t *data, spec_args_t *sargs)
-{
-	data_t *ref;
-
-	xassert(sargs->magic == MAGIC_SPEC_ARGS);
-	xassert(sargs->args->magic == MAGIC_ARGS);
-	xassert(sargs->parsers);
-	xassert(sargs->parser_count > 0);
-
-	if (!data)
-		return;
-
-	if (data_get_type(data) == DATA_TYPE_LIST)
-		(void) data_list_for_each(data, _convert_list_entry, sargs);
-
-	if (data_get_type(data) != DATA_TYPE_DICT)
-		return;
-
-	if ((ref = data_key_get(data, "$ref")) &&
-	     (data_get_type(ref) == DATA_TYPE_STRING) &&
-	     !xstrncmp(data_get_string(ref), TYPE_PREFIX,
-		       strlen(TYPE_PREFIX))) {
-		const parser_t *parser = NULL;
-
-		for (int i = 0; i < sargs->parser_count; i++) {
-			if (!xstrcmp(sargs->parsers[i].type_string,
-				     data_get_string(ref))) {
-				parser = &sargs->parsers[i];
-				break;
-			}
-		}
-
-		if (!parser) {
-			debug("%s: skipping unknown %s",
-			      __func__, data_get_string(data));
-			data_set_null(data);
-			return;
-		}
-
-		_set_ref(data, NULL, parser, sargs);
-	} else {
-		(void) data_dict_for_each(data, _convert_dict_entry, sargs);
-	}
-}
-
-static data_for_each_cmd_t _count_list_entry(data_t *data, void *arg)
-{
-	spec_args_t *sargs = arg;
-
-	xassert(sargs->magic == MAGIC_SPEC_ARGS);
-	xassert(sargs->args->magic == MAGIC_ARGS);
-
-	if ((data_get_type(data) == DATA_TYPE_LIST) ||
-	    (data_get_type(data) == DATA_TYPE_DICT))
-		_count_refs(data, sargs);
-
-	return DATA_FOR_EACH_CONT;
-}
-
-static void _increment_ref(const parser_t *parent, const parser_t *parser,
-			   spec_args_t *sargs)
-{
-	uint32_t parser_index;
-
-	while (parser->pointer_type)
-		parser = find_parser_by_type(parser->pointer_type);
-
-	if ((parser_index = _resolve_parser_index(parser, sargs)) != NO_VAL) {
-		sargs->references[parser_index]++;
-
-		debug4("%s: %s->%s incremented references=%u",
-		       __func__, (parent ? parent->type_string : "*" ),
-		       parser->type_string, sargs->references[parser_index]);
-	}
-}
-
-static data_for_each_cmd_t _count_dict_entry(const char *key, data_t *data,
-					     void *arg)
-{
-	spec_args_t *sargs = arg;
-
-	xassert(sargs->magic == MAGIC_SPEC_ARGS);
-	xassert(sargs->args->magic == MAGIC_ARGS);
-
-	if (!xstrcmp(key, "$ref") &&
-	    (data_get_type(data) == DATA_TYPE_STRING) &&
-	    !xstrncmp(data_get_string(data), TYPE_PREFIX,
-		      strlen(TYPE_PREFIX)))
-		_increment_ref(NULL, _resolve_parser(data_get_string(data),
-						     sargs), sargs);
-
-	if ((data_get_type(data) == DATA_TYPE_LIST) ||
-	    (data_get_type(data) == DATA_TYPE_DICT))
-		_count_refs(data, sargs);
-
-	return DATA_FOR_EACH_CONT;
-}
-
-/*
- * Find every $ref = DATA_PARSER_* and count references
- */
-static void _count_refs(data_t *data, spec_args_t *sargs)
 {
 	xassert(sargs->magic == MAGIC_SPEC_ARGS);
 	xassert(sargs->args->magic == MAGIC_ARGS);
@@ -605,33 +504,9 @@ static void _count_refs(data_t *data, spec_args_t *sargs)
 		return;
 
 	if (data_get_type(data) == DATA_TYPE_DICT)
-		(void) data_dict_for_each(data, _count_dict_entry, sargs);
+		(void) data_dict_for_each(data, _convert_dict_entry, sargs);
 	else if (data_get_type(data) == DATA_TYPE_LIST)
-		(void) data_list_for_each(data, _count_list_entry, sargs);
-}
-
-static void _count_parser_refs(spec_args_t *sargs)
-{
-	xassert(sargs->magic == MAGIC_SPEC_ARGS);
-	xassert(sargs->args->magic == MAGIC_ARGS);
-	xassert(sargs->parsers);
-	xassert(sargs->parser_count > 0);
-
-	for (int ip = 0; ip < sargs->parser_count; ip++) {
-		const parser_t *parser = &sargs->parsers[ip];
-
-		if ((parser->model != PARSER_MODEL_ARRAY) ||
-		    !parser->field_count)
-			continue;
-
-		for (int i = 0; i < parser->field_count; i++) {
-			const parser_t *pchild =
-				find_parser_by_type(parser->fields[i].type);
-
-			if (pchild)
-				_increment_ref(parser, pchild, sargs);
-		}
-	}
+		(void) data_list_for_each(data, _convert_list_entry, sargs);
 }
 
 static data_t *_add_param(data_t *param, const char *name,
@@ -652,8 +527,7 @@ static data_t *_add_param(data_t *param, const char *name,
 	data_set_string(data_key_set(param, "style"),
 			(in_path ? "simple" : "form"));
 	data_set_bool(data_key_set(param, "explode"), false);
-	if (deprecated)
-		data_set_bool(data_key_set(param, "deprecated"), true);
+	data_set_bool(data_key_set(param, "deprecated"), deprecated);
 	data_set_bool(data_key_set(param, "allowEmptyValue"), allow_empty);
 	data_set_bool(data_key_set(param, "allowReserved"), false);
 	if (desc)
@@ -677,8 +551,8 @@ static void _add_param_eflags(data_t *params, const parser_t *parser,
 		if (!bit->hidden)
 			_add_param(data_set_dict(data_list_append(params)),
 				   bit->name, OPENAPI_FORMAT_BOOL, true,
-				   bit->description, bit->deprecated, false,
-				   args);
+				   bit->description,
+				   IS_FLAG_BIT_DEPRECATED(bit), false, args);
 	}
 }
 
@@ -701,8 +575,7 @@ static void _add_param_linked(data_t *params, const parser_t *fp,
 	}
 
 	/* resolve out pointer type to first non-pointer */
-	while (p->pointer_type)
-		p = find_parser_by_type(p->pointer_type);
+	p = unalias_parser(p);
 
 	if (p->model == PARSER_MODEL_ARRAY) {
 		/* no way to parse an dictionary/object currently */
@@ -712,7 +585,8 @@ static void _add_param_linked(data_t *params, const parser_t *fp,
 	schema = _add_param(data_set_dict(data_list_append(params)), fp->key,
 			    OPENAPI_FORMAT_STRING,
 			    (p->obj_openapi == OPENAPI_FORMAT_BOOL),
-			    fp->obj_desc, fp->deprecated, fp->required, args);
+			    fp->obj_desc, IS_PARSER_DEPRECATED(fp),
+			    fp->required, args);
 
 	if (fp->model == PARSER_MODEL_ARRAY_LINKED_FIELD)
 		fp = find_parser_by_type(fp->type);
@@ -724,17 +598,24 @@ static void _add_param_linked(data_t *params, const parser_t *fp,
 static data_for_each_cmd_t _foreach_path_method_ref(data_t *ref, void *arg)
 {
 	spec_args_t *args = arg;
-	const parser_t *parser;
+	const parser_t *parser = NULL;
 
-	if (!(parser = _resolve_parser(data_get_string(ref), args))) {
+	for (int i = 0; i < args->parser_count; i++) {
+		if (!xstrcmp(args->parsers[i].type_string,
+			     data_get_string(ref))) {
+			parser = &args->parsers[i];
+			break;
+		}
+	}
+
+	if (!parser) {
 		error("%s: Unable to find parser for $ref = %s",
 		      __func__, data_get_string(ref));
 		return DATA_FOR_EACH_FAIL;
 	}
 
 	/* auto-dereference pointers to avoid unneeded resolution failures */
-	if (parser->model == PARSER_MODEL_PTR)
-		parser = find_parser_by_type(parser->pointer_type);
+	parser = unalias_parser(parser);
 
 	if (parser->model != PARSER_MODEL_ARRAY) {
 		error("$ref parameters must be an array parser");
@@ -864,8 +745,6 @@ static data_for_each_cmd_t _foreach_join_path(const char *key, data_t *data,
 	data_t *path = data_key_set(args->paths, key);
 
 	data_move(path, data);
-	_count_refs(path, args);
-	_count_parser_refs(args);
 	_replace_refs(path, args);
 
 	return DATA_FOR_EACH_CONT;
@@ -892,13 +771,10 @@ extern int data_parser_p_specify(args_t *args, data_t *spec)
 			     OPENAPI_SCHEMAS_PATH);
 
 	get_parsers(&sargs.parsers, &sargs.parser_count);
-	sargs.references = xcalloc(sargs.parser_count,
-				   sizeof(*sargs.references));
 
 	(void) data_dict_for_each(sargs.paths, _foreach_path, &sargs);
 	(void) data_dict_for_each(sargs.new_paths, _foreach_join_path, &sargs);
 	FREE_NULL_DATA(sargs.new_paths);
-	xfree(sargs.references);
 
 	return SLURM_SUCCESS;
 }
@@ -920,60 +796,37 @@ extern void set_openapi_schema(data_t *dst, const parser_t *parser,
 
 	get_parsers(&sargs.parsers, &sargs.parser_count);
 
-	(void) _set_openapi_parse(dst, parser, &sargs, NULL, false);
+	(void) _set_openapi_parse(dst, parser, &sargs, NULL);
 }
 
 extern int data_parser_p_increment_reference(args_t *args,
 					     data_parser_type_t type,
-					     refs_ptr_t **references_ptr)
+					     void **references_ptr)
 {
-	spec_args_t sargs = {
-		.magic = MAGIC_SPEC_ARGS,
-		.args = args,
-	};
-	refs_ptr_t *refs = *references_ptr;
-	const parser_t *parser;
-
-	get_parsers(&sargs.parsers, &sargs.parser_count);
-
-	if (!refs) {
-		refs = *references_ptr = xmalloc(sizeof(*refs));
-		refs->magic = MAGIC_REFS_PTR;
-		refs->references =
-			xcalloc(sargs.parser_count, sizeof(*refs->references));
-	}
-
-	if (!(parser = find_parser_by_type(type)))
-		return ESLURM_DATA_INVALID_PARSER;
-
-	xassert(refs->magic == MAGIC_REFS_PTR);
 	xassert(args->magic == MAGIC_ARGS);
 	xassert(type > DATA_PARSER_TYPE_INVALID);
 	xassert(type < DATA_PARSER_TYPE_MAX);
-	xassert(sargs.parser_count > 0);
+	xassert(!*references_ptr ||
+		(*references_ptr == OPENAPI_REF_PLACEHOLDER));
 
-	sargs.references = refs->references,
-	_increment_ref(NULL, parser, &sargs);
+	/* refs are not tracked in v40 */
+	*references_ptr = OPENAPI_REF_PLACEHOLDER;
 
 	return SLURM_SUCCESS;
 }
 
 extern int data_parser_p_populate_schema(args_t *args, data_parser_type_t type,
-					 refs_ptr_t **references_ptr,
-					 data_t *dst, data_t *schemas)
+					 void **references_ptr, data_t *dst,
+					 data_t *schemas)
 {
-#ifndef NDEBUG
-	refs_ptr_t *refs = *references_ptr;
-#endif
 	spec_args_t sargs = {
 		.magic = MAGIC_SPEC_ARGS,
 		.args = args,
 		.schemas = schemas,
-		.references = (*references_ptr)->references,
 	};
 	const parser_t *parser;
 
-	xassert(refs && refs->magic == MAGIC_REFS_PTR);
+	xassert(*references_ptr == OPENAPI_REF_PLACEHOLDER);
 	xassert(args->magic == MAGIC_ARGS);
 	xassert(type > DATA_PARSER_TYPE_INVALID);
 	xassert(type < DATA_PARSER_TYPE_MAX);
@@ -991,21 +844,17 @@ extern int data_parser_p_populate_schema(args_t *args, data_parser_type_t type,
 extern int data_parser_p_populate_parameters(args_t *args,
 					     data_parser_type_t parameter_type,
 					     data_parser_type_t query_type,
-					     refs_ptr_t **references_ptr,
-					     data_t *dst, data_t *schemas)
+					     void **references_ptr, data_t *dst,
+					     data_t *schemas)
 {
-#ifndef NDEBUG
-	refs_ptr_t *refs = *references_ptr;
-#endif
 	spec_args_t sargs = {
 		.magic = MAGIC_SPEC_ARGS,
 		.args = args,
 		.schemas = schemas,
-		.references = (*references_ptr)->references,
 	};
 	const parser_t *param_parser = NULL, *query_parser = NULL;
 
-	xassert(refs && refs->magic == MAGIC_REFS_PTR);
+	xassert(*references_ptr == OPENAPI_REF_PLACEHOLDER);
 	xassert(args->magic == MAGIC_ARGS);
 	xassert(!parameter_type || (parameter_type > DATA_PARSER_TYPE_INVALID));
 	xassert(!parameter_type || (parameter_type < DATA_PARSER_TYPE_MAX));
@@ -1020,16 +869,14 @@ extern int data_parser_p_populate_parameters(args_t *args,
 	sargs.path_params = data_set_dict(data_new());
 
 	if (parameter_type &&
-	    !(param_parser = find_parser_by_type(parameter_type)))
+	    !(param_parser =
+		      unalias_parser(find_parser_by_type(parameter_type))))
 		return ESLURM_DATA_INVALID_PARSER;
-	if (query_type && !(query_parser = find_parser_by_type(query_type)))
+	if (query_type &&
+	    !(query_parser = unalias_parser(find_parser_by_type(query_type))))
 		return ESLURM_DATA_INVALID_PARSER;
 
 	if (param_parser) {
-		while (param_parser->pointer_type)
-			param_parser =
-				find_parser_by_type(param_parser->pointer_type);
-
 		if (param_parser->model != PARSER_MODEL_ARRAY)
 			fatal_abort("parameters must be an array parser");
 
@@ -1047,10 +894,6 @@ extern int data_parser_p_populate_parameters(args_t *args,
 					  &sargs);
 	}
 	if (query_parser) {
-		while (query_parser->pointer_type)
-			query_parser =
-				find_parser_by_type(query_parser->pointer_type);
-
 		if (query_parser->model != PARSER_MODEL_ARRAY)
 			fatal_abort("parameters must be an array parser");
 
@@ -1070,19 +913,12 @@ extern int data_parser_p_populate_parameters(args_t *args,
 }
 
 extern void data_parser_p_release_references(args_t *args,
-					     refs_ptr_t **references_ptr)
+					     void **references_ptr)
 {
-	refs_ptr_t *refs = *references_ptr;
-
 	xassert(args->magic == MAGIC_ARGS);
 
-	if (!refs)
-		return;
-
+	/* refs are not tracked in v40 */
+	xassert(!*references_ptr ||
+		(*references_ptr == OPENAPI_REF_PLACEHOLDER));
 	*references_ptr = NULL;
-
-	xassert(refs->magic == MAGIC_REFS_PTR);
-	xfree(refs->references);
-	refs->magic = ~MAGIC_REFS_PTR;
-	xfree(refs);
 }
