@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  jobcomp_common.c - common functions for jobcomp plugins
  *****************************************************************************
- *  Copyright (C) 2022 SchedMD LLC
- *  Written by Alejandro Sanchez <alex@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -42,6 +41,16 @@
 #include "src/plugins/jobcomp/common/jobcomp_common.h"
 #include "src/slurmctld/slurmctld.h"
 
+static bool _valid_date_format(char *date_str)
+{
+	if (!date_str || !*date_str ||
+	    !xstrcasecmp(date_str, "unknown") ||
+	    !xstrcasecmp(date_str, "none"))
+		return false;
+
+	return true;
+}
+
 /*
  * Open jobcomp state file, or backup if necessary.
  *
@@ -76,54 +85,6 @@ extern buf_t *jobcomp_common_load_state_file(char *state_file)
 	return buf;
 }
 
-extern void jobcomp_common_write_state_file(buf_t *buffer, char *state_file)
-{
-	int fd;
-	bool do_close = true;
-	char *reg_file = NULL, *new_file = NULL, *old_file = NULL;
-	char *tmp_str = NULL;
-
-	xstrfmtcat(reg_file, "%s/%s", slurm_conf.state_save_location,
-		   state_file);
-	xstrfmtcat(old_file, "%s.old", reg_file);
-	xstrfmtcat(new_file, "%s.new", reg_file);
-
-	if ((fd = creat(new_file, 0600)) < 0) {
-		xstrfmtcat(tmp_str, "creating");
-		goto rwfail;
-	}
-
-	xstrfmtcat(tmp_str, "writing");
-	safe_write(fd, get_buf_data(buffer), get_buf_offset(buffer));
-	xfree(tmp_str);
-
-	do_close = false;
-	if (fsync_and_close(fd, state_file))
-		goto rwfail;
-
-	(void) unlink(old_file);
-	if (link(reg_file, old_file))
-		debug2("unable to create link for %s -> %s: %m",
-		       reg_file, old_file);
-
-	(void) unlink(reg_file);
-	if (link(new_file, reg_file))
-		debug2("unable to create link for %s -> %s: %m",
-		       new_file, reg_file);
-
-rwfail:
-	if (tmp_str)
-		error("Can't save state, error %s file %s: %m",
-		      tmp_str, new_file);
-	if (do_close && fsync_and_close(fd, state_file))
-		;
-	(void) unlink(new_file);
-	xfree(old_file);
-	xfree(reg_file);
-	xfree(new_file);
-	xfree(tmp_str);
-}
-
 extern data_t *jobcomp_common_job_record_to_data(job_record_t *job_ptr) {
 	char start_str[32], end_str[32], time_str[32];
 	char *usr_str = NULL, *grp_str = NULL, *state_string = NULL;
@@ -207,8 +168,10 @@ extern data_t *jobcomp_common_job_record_to_data(job_record_t *job_ptr) {
 	data_set_int(data_key_set(record, "user_id"), job_ptr->user_id);
 	data_set_string(data_key_set(record, "groupname"), grp_str);
 	data_set_int(data_key_set(record, "group_id"), job_ptr->group_id);
-	data_set_string(data_key_set(record, "@start"), start_str);
-	data_set_string(data_key_set(record, "@end"), end_str);
+	if (_valid_date_format(start_str))
+		data_set_string(data_key_set(record, "@start"), start_str);
+	if (_valid_date_format(end_str))
+		data_set_string(data_key_set(record, "@end"), end_str);
 	data_set_int(data_key_set(record, "elapsed"), elapsed_time);
 	data_set_string(data_key_set(record, "partition"), job_ptr->partition);
 	data_set_string(data_key_set(record, "alloc_node"),
@@ -245,22 +208,32 @@ extern data_t *jobcomp_common_job_record_to_data(job_record_t *job_ptr) {
 			     job_ptr->het_job_offset);
 	}
 
+	if ((job_ptr->priority != NO_VAL) && (job_ptr->priority != INFINITE))
+		data_set_int(data_key_set(record, "priority"),
+			     job_ptr->priority);
+
 	if (job_ptr->details && job_ptr->details->submit_time) {
 		parse_time_make_str_utc(&job_ptr->details->submit_time,
 					time_str, sizeof(time_str));
-		data_set_string(data_key_set(record, "@submit"), time_str);
+		if (_valid_date_format(time_str))
+			data_set_string(data_key_set(record, "@submit"),
+					time_str);
 	}
 
 	if (job_ptr->details && job_ptr->details->begin_time) {
 		parse_time_make_str_utc(&job_ptr->details->begin_time, time_str,
 					sizeof(time_str));
-		data_set_string(data_key_set(record, "@eligible"), time_str);
+		if (_valid_date_format(time_str))
+			data_set_string(data_key_set(record, "@eligible"),
+					time_str);
 		if (job_ptr->start_time) {
 			int64_t queue_wait = (int64_t)difftime(
 				job_ptr->start_time,
 				job_ptr->details->begin_time);
-			data_set_int(data_key_set(record, "@queue_wait"),
-				     queue_wait);
+			if (queue_wait >= 0)
+				data_set_int(data_key_set(record,
+							  "@queue_wait"),
+							  queue_wait);
 		}
 	}
 
