@@ -107,7 +107,7 @@ extern int mcs_p_set_mcs_label(job_record_t *job_ptr, char *label)
 			rc = SLURM_ERROR;
 	} else {
 		if ((slurm_mcs_get_enforced() == 0) && job_ptr->details &&
-		    (job_ptr->details->whole_node != WHOLE_NODE_MCS))
+		    !(job_ptr->details->whole_node & WHOLE_NODE_MCS))
 			;
 		else
 			job_ptr->mcs_label = xstrdup(job_ptr->account);
