@@ -164,7 +164,7 @@ int pmixp_coll_ring_unpack(buf_t *buf, pmixp_coll_type_t *type,
 	}
 	*nr = nprocs;
 
-	procs = xmalloc(sizeof(pmix_proc_t) * nprocs);
+	procs = xcalloc(nprocs, sizeof(pmix_proc_t));
 	*r = procs;
 
 	/* 3. get namespace/rank of particular process */
@@ -255,7 +255,7 @@ static int _ring_forward_data(pmixp_coll_ring_ctx_t *coll_ctx, uint32_t contrib_
 	hdr.seq = coll_ctx->seq;
 	hdr.hop_seq = hop_seq;
 	hdr.contrib_id = contrib_id;
-	pmixp_ep_t *ep = (pmixp_ep_t*)xmalloc(sizeof(*ep));
+	pmixp_ep_t *ep = xmalloc(sizeof(*ep));
 	pmixp_coll_ring_cbdata_t *cbdata = NULL;
 	uint32_t offset = 0;
 	buf_t *buf = _get_fwd_buf(coll_ctx);
@@ -281,7 +281,8 @@ static int _ring_forward_data(pmixp_coll_ring_ctx_t *coll_ctx, uint32_t contrib_
 
 	/* insert payload to buf */
 	offset = get_buf_offset(buf);
-	pmixp_server_buf_reserve(buf, size);
+	if ((rc = try_grow_buf_remaining(buf, size)))
+		goto exit;
 	memcpy(get_buf_data(buf) + offset, data, size);
 	set_buf_offset(buf, offset + size);
 
@@ -503,7 +504,7 @@ int pmixp_coll_ring_init(pmixp_coll_t *coll, hostlist_t **hl)
 		coll_ctx->contrib_prev = 0;
 		coll_ctx->state = PMIXP_COLL_RING_SYNC;
 		// TODO bit vector
-		coll_ctx->contrib_map = xmalloc(sizeof(bool) * coll->peers_cnt);
+		coll_ctx->contrib_map = xcalloc(coll->peers_cnt, sizeof(bool));
 	}
 
 	return SLURM_SUCCESS;
@@ -535,14 +536,9 @@ inline static int _pmixp_coll_contrib(pmixp_coll_ring_ctx_t *coll_ctx,
 	coll->ts = time(NULL);
 
 	/* save contribution */
-	if (!size_buf(coll_ctx->ring_buf)) {
-		grow_buf(coll_ctx->ring_buf, size * coll->peers_cnt);
-	} else if(remaining_buf(coll_ctx->ring_buf) < size) {
-		uint32_t new_size = size_buf(coll_ctx->ring_buf) + size *
-			_ring_remain_contrib(coll_ctx);
-		grow_buf(coll_ctx->ring_buf, new_size);
-	}
-	grow_buf(coll_ctx->ring_buf, size);
+	if (try_grow_buf_remaining(coll_ctx->ring_buf, size))
+		return SLURM_ERROR;
+
 	data_ptr = get_buf_data(coll_ctx->ring_buf) +
 		get_buf_offset(coll_ctx->ring_buf);
 	memcpy(data_ptr, data, size);
