@@ -139,7 +139,7 @@ static void _load_config(void)
 static void _compute_start_times(void)
 {
 	int j, rc = SLURM_SUCCESS, job_cnt = 0;
-	List job_queue;
+	list_t *job_queue = NULL;
 	job_queue_rec_t *job_queue_rec;
 	job_record_t *job_ptr;
 	part_record_t *part_ptr;
@@ -203,7 +203,8 @@ static void _compute_start_times(void)
 				       min_nodes, max_nodes, req_nodes,
 				       SELECT_MODE_WILL_RUN,
 				       NULL, NULL,
-				       &resv_exc);
+				       &resv_exc,
+				       NULL);
 		if (rc == SLURM_SUCCESS) {
 			last_job_update = now;
 			if (job_ptr->time_limit == INFINITE)
