@@ -1383,10 +1383,16 @@ static int _build_tasks_per_node_sock(struct job_resources *job_res,
 		tasks_per_node_socket[i] = xcalloc(sock_cnt, sizeof(uint32_t));
 		if (tres_mc_ptr->ntasks_per_node) {
 			task_per_node_limit = tres_mc_ptr->ntasks_per_node;
+			cpus_per_task = MAX(1, job_res->cpus[job_node_inx] /
+						       job_res->tasks_per_node
+							       [job_node_inx]);
 		} else if (job_res->tasks_per_node &&
 			   job_res->tasks_per_node[job_node_inx]) {
 			task_per_node_limit =
 				job_res->tasks_per_node[job_node_inx];
+			cpus_per_task =
+				MAX(1, job_res->cpus[job_node_inx] /
+				    job_res->tasks_per_node[job_node_inx]);
 		} else {
 			/*
 			 * NOTE: We should never get here.
