@@ -41,10 +41,13 @@
 #include "switch_hpe_slingshot.h"
 #include "rest.h"
 
+#include "src/common/job_record.h"
+#include "src/common/node_conf.h"
+
 /* Set this to true if VNI table is re-sized and loses some bits */
 static bool lost_vnis = false;
 /* Number of free VNIs */
-static int free_vnis = 0;
+int free_vnis = 0;
 
 /*
  * Set up slingshot_config defaults
@@ -299,92 +302,6 @@ out:
 	return true;
 }
 
-/*
- * Parse the "jlope_url" token, with format "jlope_url=<url>"
- */
-static bool _config_jlope_url(const char *token, char *arg)
-{
-	if (!arg)
-		goto err;
-	slingshot_config.jlope_url = xstrdup(arg);
-
-	log_flag(SWITCH, "[token=%s]: jlope_url %s",
-		 token, slingshot_config.jlope_url);
-	return true;
-err:
-	error("Invalid jlope_url token '%s' (example 'jlope_url=https://api-gw-service-nmn.local/apis/jackaloped')",
-		token);
-	return false;
-}
-
-/*
- * Parse the "jlope_auth" token, with format "jlope_auth={BASIC,OAUTH}"
- */
-static bool _config_jlope_auth(const char *token, char *arg)
-{
-	if (!arg)
-		goto err;
-	if (!xstrcasecmp(arg, SLINGSHOT_AUTH_BASIC_STR))
-		slingshot_config.jlope_auth = SLINGSHOT_AUTH_BASIC;
-	else if (!xstrcasecmp(arg, SLINGSHOT_AUTH_OAUTH_STR))
-		slingshot_config.jlope_auth = SLINGSHOT_AUTH_OAUTH;
-	else
-		goto err;
-
-	log_flag(SWITCH, "[token=%s]: jlope_auth %d",
-		 token, slingshot_config.jlope_auth);
-	return true;
-err:
-	error("Invalid jlope_auth token '%s' (example 'jlope_auth={BASIC,OAUTH}')",
-		token);
-	return false;
-}
-
-/*
- * Parse the "jlope_authdir" token, with format "jlope_authdir=<dirpath>"
- */
-static bool _config_jlope_authdir(const char *token, char *arg)
-{
-	struct stat statbuf;
-
-	if (!arg)
-		goto err;
-	if (stat(arg, &statbuf) != 0 || !S_ISDIR(statbuf.st_mode)) {
-		error("jlope_authdir directory '%s' is not a directory", arg);
-		return false;
-	}
-	slingshot_config.jlope_authdir = xstrdup(arg);
-
-	log_flag(SWITCH, "[token=%s]: jlope_authdir %s",
-		 token, slingshot_config.jlope_authdir);
-	return true;
-err:
-	error("Invalid jlope_authdir token '%s' (example 'jlope_authdir=/etc/wlm-client-auth')",
-		token);
-	return false;
-}
-
-/*
- * If jlope_url is set, set up default values for jlope_auth{dir}
- * (if not already set)
- */
-static void _config_jlope_defaults(void)
-{
-	if (!slingshot_config.jlope_url)
-		return;
-	if (slingshot_config.jlope_auth == SLINGSHOT_AUTH_NONE)
-		slingshot_config.jlope_auth = SLINGSHOT_AUTH_OAUTH;
-	if (!slingshot_config.jlope_authdir) {
-		if (slingshot_config.jlope_auth == SLINGSHOT_AUTH_OAUTH)
-			slingshot_config.jlope_authdir =
-					xstrdup(SLINGSHOT_JLOPE_AUTH_OAUTH_DIR);
-		else if (slingshot_config.jlope_auth == SLINGSHOT_AUTH_BASIC)
-			slingshot_config.jlope_authdir =
-					xstrdup(SLINGSHOT_JLOPE_AUTH_BASIC_DIR);
-	}
-	xassert(slingshot_config.jlope_authdir);
-}
-
 /*
  * Parse the "hwcoll_addrs_per_job" token, with format
  * "hwcoll_addrs_per_job=<number>"
@@ -646,8 +563,6 @@ static void _print_limits(slingshot_limits_set_t *limits)
  */
 extern void slingshot_free_config(void)
 {
-	xfree(slingshot_config.jlope_url);
-	xfree(slingshot_config.jlope_authdir);
 	xfree(slingshot_config.fm_url);
 	xfree(slingshot_config.fm_authdir);
 }
@@ -671,12 +586,6 @@ extern bool slingshot_setup_config(const char *switch_params)
 	const size_t size_adjust_limits = sizeof(adjust_limits) - 1;
 	const char no_adjust_limits[] = "no_adjust_limits";
 	const size_t size_no_adjust_limits = sizeof(no_adjust_limits) - 1;
-	const char jlope_url[] = "jlope_url";
-	const size_t size_jlope_url = sizeof(jlope_url) - 1;
-	const char jlope_auth[] = "jlope_auth";
-	const size_t size_jlope_auth = sizeof(jlope_auth) - 1;
-	const char jlope_authdir[] = "jlope_authdir";
-	const size_t size_jlope_authdir = sizeof(jlope_authdir) - 1;
 	const char hwcoll_addrs_per_job[] = "hwcoll_addrs_per_job";
 	const size_t size_hwcoll_addrs_per_job =
                                         sizeof(hwcoll_addrs_per_job) - 1;
@@ -692,7 +601,6 @@ extern bool slingshot_setup_config(const char *switch_params)
 	uint16_t vni_min = slingshot_state.vni_min;
 	uint16_t vni_max = slingshot_state.vni_max;
 
-	log_flag(SWITCH, "switch_params=%s", switch_params);
 	/*
 	 * Handle SwitchParameters values (separated by commas):
 	 *
@@ -707,9 +615,6 @@ extern bool slingshot_setup_config(const char *switch_params)
 	 *   {no_}adjust_limits: {don't} adjust resource reservations
 	 *     for each NIC by subtracting resources already
 	 *     used/reserved by system services
-	 *   jlope_url=<url>: use URL for jackaloped REST requests
-	 *   jlope_auth="BASIC|OAUTH": jackaloped REST API authentication type
-	 *   jlope_authdir=<dir>: jackaloped authentication info directory
 	 *     (i.e. /etc/jackaloped for BASIC, /etc/wlm-client-auth for OAUTH)
 	 *   hwcoll_addrs_per_job=<number>: allocate <number> of Slingshot
          *     hardware collectives multicast addresses per job
@@ -743,6 +648,7 @@ extern bool slingshot_setup_config(const char *switch_params)
 			goto err;
 		goto out;
 	}
+	log_flag(SWITCH, "switch_params=%s", switch_params);
 
 	params = xstrdup(switch_params);
 	for (token = strtok_r(params, ",", &save_ptr); token;
@@ -772,20 +678,6 @@ extern bool slingshot_setup_config(const char *switch_params)
 					 size_no_adjust_limits)) {
 			slingshot_config.flags &=
 					~(SLINGSHOT_FLAGS_ADJUST_LIMITS);
-		} else if (!xstrncasecmp(token, jlope_url, size_jlope_url)) {
-			if (!_config_jlope_url(token, arg))
-				goto err;
-		/*
-		 * NOTE: jlope_authdir needs to come before jlope_auth
-		 * since jlope_auth is a prefix of jlope_authdir
-		 */
-		} else if (!xstrncasecmp(token, jlope_authdir,
-			   size_jlope_authdir)) {
-			if (!_config_jlope_authdir(token, arg))
-				goto err;
-		} else if (!xstrncasecmp(token, jlope_auth, size_jlope_auth)) {
-			if (!_config_jlope_auth(token, arg))
-				goto err;
 		} else if (!xstrncasecmp(token, hwcoll_addrs_per_job,
                                          size_hwcoll_addrs_per_job)) {
 			if (!_config_hwcoll_addrs_per_job(token, arg))
@@ -812,14 +704,9 @@ extern bool slingshot_setup_config(const char *switch_params)
 				goto err;
 		}
 	}
-	/* If jlope_url is set, set up default values for jlope_auth{dir} */
-	_config_jlope_defaults();
 	/* If fm_url is set, set up default values for fm_auth{dir} */
 	_config_fm_defaults();
 
-	/* Set up connection to jackaloped */
-	if (!slingshot_init_instant_on())
-		goto err;
 	/* Set up connection to fabric manager */
 	if (!slingshot_init_collectives())
 		goto err;
@@ -828,9 +715,6 @@ out:
 	debug("single_node_vni=%d job_vni=%d tcs=%#x flags=%#x",
 	      slingshot_config.single_node_vni, slingshot_config.job_vni,
 	      slingshot_config.tcs, slingshot_config.flags);
-	debug("jlope_url=%s jlope_auth=%u jlope_authdir=%s",
-	      slingshot_config.jlope_url, slingshot_config.jlope_auth,
-	      slingshot_config.jlope_authdir);
 	debug("fm_url=%s fm_auth=%u fm_authdir=%s hwcoll_addrs_per_job=%d hwcoll_num_nodes=%d",
               slingshot_config.fm_url, slingshot_config.fm_auth,
 	      slingshot_config.fm_authdir,
@@ -1040,7 +924,7 @@ err:
  *   max_<NIC_resource>: maximum value for resource
  */
 static bool _parse_network_token(const char *token, bool is_job,
-				 slingshot_jobinfo_t *job,
+				 slingshot_stepinfo_t *job,
 				 bool *job_vni, bool *single_node_vni,
 				 bool *no_vni)
 {
@@ -1058,6 +942,8 @@ static bool _parse_network_token(const char *token, bool is_job,
 	size_t no_adjust_limits_siz = sizeof(no_adjust_limits_str) - 1;
 	char rdzv_get_str[] = "disable_rdzv_get";
 	size_t rdzv_get_siz = sizeof(rdzv_get_str) - 1;
+	char hwcoll_str[] = "hwcoll";
+	size_t hwcoll_siz = sizeof(hwcoll_siz) - 1;
 	char tcs_str[] = "tcs";
 	size_t tcs_siz = sizeof(tcs_str) - 1;
 
@@ -1103,6 +989,8 @@ static bool _parse_network_token(const char *token, bool is_job,
 		job->flags &= ~(SLINGSHOT_FLAGS_ADJUST_LIMITS);
 	} else if (!xstrncmp(token, rdzv_get_str, rdzv_get_siz)) {
 		job->flags |= SLINGSHOT_FLAGS_DISABLE_RDZV_GET;
+	} else if (!xstrncmp(token, hwcoll_str, hwcoll_siz)) {
+		;
 	} else if (!xstrncmp(token, tcs_str, tcs_siz)) {
 		if (is_job)
 			return _config_tcs(token, arg, &job->tcs);
@@ -1114,12 +1002,12 @@ static bool _parse_network_token(const char *token, bool is_job,
 }
 
 /*
- * Set up passed-in slingshot_jobinfo_t based on values in srun/sbatch/salloc
+ * Set up passed-in slingshot_stepinfo_t based on values in srun/sbatch/salloc
  * --network parameters.  Return true on successful parsing, false otherwise.
  */
 static bool _setup_network_params(const char *network_params,
 				  const char *job_network_params,
-				  slingshot_jobinfo_t *job,
+				  slingshot_stepinfo_t *job,
 				  bool *job_vni,
 				  bool *single_node_vni,
 				  bool *no_vni)
@@ -1182,14 +1070,71 @@ err:
 	return false;
 }
 
+extern bool slingshot_setup_job_vni_pool(job_record_t *job_ptr)
+{
+	int alloc_vnis = 0;
+	bool alloc_job_vni, alloc_single_node_vni, no_vni;
+	slingshot_stepinfo_t stepinfo = {0};
+	slingshot_jobinfo_t *jobinfo = NULL;
+
+	if (!job_ptr->switch_jobinfo)
+		job_ptr->switch_jobinfo = xmalloc(sizeof(slingshot_jobinfo_t));
+
+	/*
+	 * If --network specified, add any depth, limits,
+	 * {job,single_node,no}_vni settings
+	 * Copy configured Slingshot limits to job, add any --network settings
+	 */
+	if (!_setup_network_params(NULL, job_ptr->network,
+				   &stepinfo, &alloc_job_vni,
+				   &alloc_single_node_vni, &no_vni))
+		goto err;
+
+	jobinfo = (slingshot_jobinfo_t *) job_ptr->switch_jobinfo;
+
+	/*
+	 * VNIs and traffic classes are not allocated if:
+	 * --network=no_vni is set, or single-node jobs,
+	 * unless 'single_node_vni=all' is set in the configuration,
+	 * or 'single_node_vni=user' is set in the configuration and
+	 *    'srun --network=single_node_vni' is used
+	 */
+	jobinfo->num_vnis = 0;
+	if (!no_vni &&
+	    ((job_ptr->node_cnt > 1) || alloc_single_node_vni)) {
+		alloc_vnis = job_ptr->node_cnt;
+		if (alloc_job_vni)
+			alloc_vnis++;
+	}
+
+	jobinfo->vnis = xcalloc(alloc_vnis, sizeof(uint16_t));
+	for (int i = 0; i < alloc_vnis; i++) {
+		if (!_alloc_vni(&jobinfo->vnis[i]))
+			goto err;
+		jobinfo->num_vnis++;
+	}
+
+	return true;
+
+err:
+	if (jobinfo) {
+		for (int i = 0; i < jobinfo->num_vnis; i++)
+			_free_vni(jobinfo->vnis[i]);
+		slingshot_free_jobinfo(jobinfo);
+	}
+	job_ptr->switch_jobinfo = NULL;
+
+	return false;
+}
+
 /*
- * Set up slingshot_jobinfo_t struct with VNIs, and CXI limits,
+ * Set up slingshot_stepinfo_t struct with VNIs, and CXI limits,
  * based on configured limits as well as any specified with
  * the --network option
  * Return true on success, false if VNI cannot be allocated,
  * or --network parameters have syntax errors
  */
-extern bool slingshot_setup_job_step_vni(slingshot_jobinfo_t *job, int node_cnt,
+extern bool slingshot_setup_job_step_vni(slingshot_stepinfo_t *job, int node_cnt,
 					 uint32_t job_id,
 					 const char *network_params,
 					 const char *job_network_params)
@@ -1257,11 +1202,33 @@ err:
 	return false;
 }
 
+/*
+ * Free job VNI pool (if any)
+ */
+extern void slingshot_free_job_vni_pool(slingshot_jobinfo_t *job)
+{
+	/* slingshot_config is only initialized on the ctld and stepmgr */
+	if (!running_in_slurmctld() && !active_outside_ctld)
+		return;
+
+	if (!job)
+		return;
+
+	for (int i = 0; i < job->num_vnis; i++) {
+		_free_vni(job->vnis[i]);
+		debug("free vni=%hu free_vnis=%d", job->vnis[0], free_vnis);
+	}
+}
+
 /*
  * Free job-step VNI (if any)
  */
-extern void slingshot_free_job_step_vni(slingshot_jobinfo_t *job)
+extern void slingshot_free_job_step_vni(slingshot_stepinfo_t *job)
 {
+	/* slingshot_config is only initialized on the ctld and stepmgr */
+	if (!running_in_slurmctld() && !active_outside_ctld)
+		return;
+
 	/* Second VNI is a job VNI - don't free until job is complete */
 	if (job->vnis && (job->num_vnis > 0)) {
 		_free_vni(job->vnis[0]);
@@ -1277,3 +1244,32 @@ extern void slingshot_free_job_vni(uint32_t job_id)
 	uint16_t vni = _free_job_vni(job_id);
 	debug("free job_vni=%hu free_vnis=%d", vni, free_vnis);
 }
+
+extern void slingshot_free_jobinfo(slingshot_jobinfo_t *jobinfo)
+{
+	if (!jobinfo)
+		return;
+
+	xfree(jobinfo->vnis);
+	xfree(jobinfo);
+}
+
+extern int slingshot_update_config(slingshot_jobinfo_t *jobinfo)
+{
+	free_vnis = 0;
+	bit_set_all(slingshot_state.vni_table);
+	for (int i = 0; i < jobinfo->num_vnis; i++) {
+		bit_clear(slingshot_state.vni_table,
+			  (jobinfo->vnis[i] - slingshot_state.vni_min));
+		free_vnis++;
+	}
+	if (slurm_conf.debug_flags & DEBUG_FLAG_SWITCH)	{
+		char *bit_str = bit_fmt_full(slingshot_state.vni_table);
+		log_flag(SWITCH, "%s: min/max: %hu/%hu free_vnis: %d bitstr: %s",
+			 __func__, slingshot_state.vni_min,
+			 slingshot_state.vni_max, free_vnis, bit_str);
+		xfree(bit_str);
+	}
+
+	return SLURM_SUCCESS;
+}
