@@ -330,9 +330,13 @@ scontrol_parse_part_options (int argc, char **argv, int *update_cnt_ptr,
 		}
 		else if (xstrncasecmp(tag, "PreemptMode", MAX(taglen, 3)) == 0) {
 			uint16_t new_mode = preempt_mode_num(val);
-			if (new_mode != NO_VAL16)
+			if (new_mode != NO_VAL16) {
+				if (new_mode & PREEMPT_MODE_GANG) {
+					error("PreemptMode=GANG is a cluster-wide option and cannot be set at partition level");
+					return SLURM_ERROR;
+				}
 				part_msg_ptr->preempt_mode = new_mode;
-			else {
+			} else {
 				error("Invalid input: %s", argv[i]);
 				return SLURM_ERROR;
 			}
@@ -513,7 +517,7 @@ scontrol_update_part (int argc, char **argv)
 
 	if (slurm_update_partition(&part_msg)) {
 		exit_code = 1;
-		return slurm_get_errno ();
+		return errno;
 	} else
 		return SLURM_SUCCESS;
 }
@@ -559,7 +563,7 @@ scontrol_create_part (int argc, char **argv)
 	if (slurm_create_partition(&part_msg)) {
 		exit_code = 1;
 		slurm_perror("Error creating the partition");
-		return slurm_get_errno ();
+		return errno;
 	} else
 		return SLURM_SUCCESS;
 }
