@@ -51,6 +51,7 @@
 #include "src/common/node_conf.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_protocol_defs.h"
+#include "src/common/state_save.h"
 #include "src/common/uid.h"
 #include "src/common/xstring.h"
 #include "src/slurmctld/front_end.h"
@@ -653,9 +654,8 @@ extern int dump_all_front_end_state(void)
 {
 #ifdef HAVE_FRONT_END
 	/* Save high-water mark to avoid buffer growth with copies */
-	static int high_buffer_size = (1024 * 1024);
-	int error_code = 0, i, log_fd;
-	char *old_file, *new_file, *reg_file;
+	static uint32_t high_buffer_size = (1024 * 1024);
+	int error_code = 0, i;
 	front_end_record_t *front_end_ptr;
 	/* Locks: Read config and node */
 	slurmctld_lock_t node_read_lock = { READ_LOCK, NO_LOCK, READ_LOCK,
@@ -678,56 +678,10 @@ extern int dump_all_front_end_state(void)
 		_dump_front_end_state(front_end_ptr, buffer);
 	}
 
-	old_file = xstrdup(slurm_conf.state_save_location);
-	xstrcat (old_file, "/front_end_state.old");
-	reg_file = xstrdup(slurm_conf.state_save_location);
-	xstrcat (reg_file, "/front_end_state");
-	new_file = xstrdup(slurm_conf.state_save_location);
-	xstrcat (new_file, "/front_end_state.new");
 	unlock_slurmctld (node_read_lock);
 
-	/* write the buffer to file */
-	lock_state_files();
-	log_fd = creat (new_file, 0600);
-	if (log_fd < 0) {
-		error ("Can't save state, error creating file %s %m", new_file);
-		error_code = errno;
-	} else {
-		int pos = 0, nwrite = get_buf_offset(buffer), amount, rc;
-		char *data = (char *)get_buf_data(buffer);
-		high_buffer_size = MAX(nwrite, high_buffer_size);
-		while (nwrite > 0) {
-			amount = write(log_fd, &data[pos], nwrite);
-			if ((amount < 0) && (errno != EINTR)) {
-				error("Error writing file %s, %m", new_file);
-				error_code = errno;
-				break;
-			}
-			nwrite -= amount;
-			pos    += amount;
-		}
-
-		rc = fsync_and_close(log_fd, "front_end");
-		if (rc && !error_code)
-			error_code = rc;
-	}
-	if (error_code)
-		(void) unlink (new_file);
-	else {	/* file shuffle */
-		(void) unlink (old_file);
-		if (link(reg_file, old_file))
-			debug4("unable to create link for %s -> %s: %m",
-			       reg_file, old_file);
-		(void) unlink (reg_file);
-		if (link(new_file, reg_file))
-			debug4("unable to create link for %s -> %s: %m",
-			       new_file, reg_file);
-		(void) unlink (new_file);
-	}
-	xfree (old_file);
-	xfree (reg_file);
-	xfree (new_file);
-	unlock_state_files ();
+	error_code = save_buf_to_state("front_end_state", buffer,
+					&high_buffer_size);
 
 	FREE_NULL_BUFFER(buffer);
 	END_TIMER2(__func__);
@@ -943,6 +897,7 @@ extern void sync_front_end_state(void)
 				      job_ptr->batch_host, job_ptr);
 				job_state_set(job_ptr, (JOB_NODE_FAIL |
 							JOB_COMPLETING));
+				job_ptr->exit_code = 1;
 			} else if (job_ptr->front_end_ptr == NULL) {
 				info("front end node %s has vanished",
 				     job_ptr->batch_host);
