@@ -271,11 +271,6 @@ extern slurm_step_ctx_t *step_ctx_create_no_alloc(
 		step_req->num_tasks,
 		0);
 
-	if (switch_g_build_stepinfo(&step_resp->switch_step,
-				    step_resp->step_layout,
-				    NULL) < 0)
-		fatal("switch_g_build_stepinfo: %m");
-
 	step_resp->job_step_id = step_id;
 
 	ctx = xmalloc(sizeof(struct slurm_step_ctx_struct));
@@ -308,7 +303,7 @@ extern slurm_step_ctx_t *step_ctx_create_no_alloc(
 extern int step_ctx_destroy(slurm_step_ctx_t *ctx)
 {
 	if ((ctx == NULL) || (ctx->magic != STEP_CTX_MAGIC)) {
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return SLURM_ERROR;
 	}
 	slurm_free_job_step_create_request_msg(ctx->step_req);
